@@ -28,6 +28,7 @@
 #include <string.h>
 
 #include "py/runtime.h"
+#include "py/binary.h"
 
 #if MICROPY_PY_FRAMEBUF
 
@@ -240,11 +241,17 @@ STATIC mp_framebuf_p_t formats[] = {
     [FRAMEBUF_MHMSB] = {mono_horiz_setpixel, mono_horiz_getpixel, mono_horiz_fill_rect},
 };
 
-static inline void setpixel(const mp_obj_framebuf_t *fb, unsigned int x, unsigned int y, uint32_t col) {
+STATIC inline void setpixel(const mp_obj_framebuf_t *fb, unsigned int x, unsigned int y, uint32_t col) {
     formats[fb->format].setpixel(fb, x, y, col);
 }
 
-static inline uint32_t getpixel(const mp_obj_framebuf_t *fb, unsigned int x, unsigned int y) {
+STATIC void setpixel_checked(const mp_obj_framebuf_t *fb, mp_int_t x, mp_int_t y, mp_int_t col, mp_int_t mask) {
+    if (mask && 0 <= x && x < fb->width && 0 <= y && y < fb->height) {
+        setpixel(fb, x, y, col);
+    }
+}
+
+STATIC inline uint32_t getpixel(const mp_obj_framebuf_t *fb, unsigned int x, unsigned int y) {
     return formats[fb->format].getpixel(fb, x, y);
 }
 
@@ -263,21 +270,21 @@ STATIC void fill_rect(const mp_obj_framebuf_t *fb, int x, int y, int w, int h, u
     formats[fb->format].fill_rect(fb, x, y, xend - x, yend - y, col);
 }
 
-STATIC mp_obj_t framebuf_make_new(const mp_obj_type_t *type, size_t n_args, size_t n_kw, const mp_obj_t *args) {
+STATIC mp_obj_t framebuf_make_new(const mp_obj_type_t *type, size_t n_args, size_t n_kw, const mp_obj_t *args_in) {
     mp_arg_check_num(n_args, n_kw, 4, 5, false);
 
     mp_obj_framebuf_t *o = mp_obj_malloc(mp_obj_framebuf_t, type);
-    o->buf_obj = args[0];
+    o->buf_obj = args_in[0];
 
     mp_buffer_info_t bufinfo;
-    mp_get_buffer_raise(args[0], &bufinfo, MP_BUFFER_WRITE);
+    mp_get_buffer_raise(args_in[0], &bufinfo, MP_BUFFER_WRITE);
     o->buf = bufinfo.buf;
 
-    o->width = mp_obj_get_int(args[1]);
-    o->height = mp_obj_get_int(args[2]);
-    o->format = mp_obj_get_int(args[3]);
+    o->width = mp_obj_get_int(args_in[1]);
+    o->height = mp_obj_get_int(args_in[2]);
+    o->format = mp_obj_get_int(args_in[3]);
     if (n_args >= 5) {
-        o->stride = mp_obj_get_int(args[4]);
+        o->stride = mp_obj_get_int(args_in[4]);
     } else {
         o->stride = o->width;
     }
@@ -305,6 +312,12 @@ STATIC mp_obj_t framebuf_make_new(const mp_obj_type_t *type, size_t n_args, size
     return MP_OBJ_FROM_PTR(o);
 }
 
+STATIC void framebuf_args(const mp_obj_t *args_in, mp_int_t *args_out, int n) {
+    for (int i = 0; i < n; ++i) {
+        args_out[i] = mp_obj_get_int(args_in[i + 1]);
+    }
+}
+
 STATIC mp_int_t framebuf_get_buffer(mp_obj_t self_in, mp_buffer_info_t *bufinfo, mp_uint_t flags) {
     (void)flags;
     mp_obj_framebuf_t *self = MP_OBJ_TO_PTR(self_in);
@@ -322,98 +335,75 @@ STATIC mp_obj_t framebuf_fill(mp_obj_t self_in, mp_obj_t col_in) {
 }
 STATIC MP_DEFINE_CONST_FUN_OBJ_2(framebuf_fill_obj, framebuf_fill);
 
-STATIC mp_obj_t framebuf_fill_rect(size_t n_args, const mp_obj_t *args) {
-    (void)n_args;
-
-    mp_obj_framebuf_t *self = MP_OBJ_TO_PTR(args[0]);
-    mp_int_t x = mp_obj_get_int(args[1]);
-    mp_int_t y = mp_obj_get_int(args[2]);
-    mp_int_t width = mp_obj_get_int(args[3]);
-    mp_int_t height = mp_obj_get_int(args[4]);
-    mp_int_t col = mp_obj_get_int(args[5]);
-
-    fill_rect(self, x, y, width, height, col);
-
+STATIC mp_obj_t framebuf_fill_rect(size_t n_args, const mp_obj_t *args_in) {
+    mp_obj_framebuf_t *self = MP_OBJ_TO_PTR(args_in[0]);
+    mp_int_t args[5]; // x, y, w, h, col
+    framebuf_args(args_in, args, 5);
+    fill_rect(self, args[0], args[1], args[2], args[3], args[4]);
     return mp_const_none;
 }
 STATIC MP_DEFINE_CONST_FUN_OBJ_VAR_BETWEEN(framebuf_fill_rect_obj, 6, 6, framebuf_fill_rect);
 
-STATIC mp_obj_t framebuf_pixel(size_t n_args, const mp_obj_t *args) {
-    mp_obj_framebuf_t *self = MP_OBJ_TO_PTR(args[0]);
-    mp_int_t x = mp_obj_get_int(args[1]);
-    mp_int_t y = mp_obj_get_int(args[2]);
+STATIC mp_obj_t framebuf_pixel(size_t n_args, const mp_obj_t *args_in) {
+    mp_obj_framebuf_t *self = MP_OBJ_TO_PTR(args_in[0]);
+    mp_int_t x = mp_obj_get_int(args_in[1]);
+    mp_int_t y = mp_obj_get_int(args_in[2]);
     if (0 <= x && x < self->width && 0 <= y && y < self->height) {
         if (n_args == 3) {
             // get
             return MP_OBJ_NEW_SMALL_INT(getpixel(self, x, y));
         } else {
             // set
-            setpixel(self, x, y, mp_obj_get_int(args[3]));
+            setpixel(self, x, y, mp_obj_get_int(args_in[3]));
         }
     }
     return mp_const_none;
 }
 STATIC MP_DEFINE_CONST_FUN_OBJ_VAR_BETWEEN(framebuf_pixel_obj, 3, 4, framebuf_pixel);
 
-STATIC mp_obj_t framebuf_hline(size_t n_args, const mp_obj_t *args) {
+STATIC mp_obj_t framebuf_hline(size_t n_args, const mp_obj_t *args_in) {
     (void)n_args;
 
-    mp_obj_framebuf_t *self = MP_OBJ_TO_PTR(args[0]);
-    mp_int_t x = mp_obj_get_int(args[1]);
-    mp_int_t y = mp_obj_get_int(args[2]);
-    mp_int_t w = mp_obj_get_int(args[3]);
-    mp_int_t col = mp_obj_get_int(args[4]);
+    mp_obj_framebuf_t *self = MP_OBJ_TO_PTR(args_in[0]);
+    mp_int_t args[4]; // x, y, w, col
+    framebuf_args(args_in, args, 4);
 
-    fill_rect(self, x, y, w, 1, col);
+    fill_rect(self, args[0], args[1], args[2], 1, args[3]);
 
     return mp_const_none;
 }
 STATIC MP_DEFINE_CONST_FUN_OBJ_VAR_BETWEEN(framebuf_hline_obj, 5, 5, framebuf_hline);
 
-STATIC mp_obj_t framebuf_vline(size_t n_args, const mp_obj_t *args) {
+STATIC mp_obj_t framebuf_vline(size_t n_args, const mp_obj_t *args_in) {
     (void)n_args;
 
-    mp_obj_framebuf_t *self = MP_OBJ_TO_PTR(args[0]);
-    mp_int_t x = mp_obj_get_int(args[1]);
-    mp_int_t y = mp_obj_get_int(args[2]);
-    mp_int_t h = mp_obj_get_int(args[3]);
-    mp_int_t col = mp_obj_get_int(args[4]);
+    mp_obj_framebuf_t *self = MP_OBJ_TO_PTR(args_in[0]);
+    mp_int_t args[4]; // x, y, h, col
+    framebuf_args(args_in, args, 4);
 
-    fill_rect(self, x, y, 1, h, col);
+    fill_rect(self, args[0], args[1], 1, args[2], args[3]);
 
     return mp_const_none;
 }
 STATIC MP_DEFINE_CONST_FUN_OBJ_VAR_BETWEEN(framebuf_vline_obj, 5, 5, framebuf_vline);
 
-STATIC mp_obj_t framebuf_rect(size_t n_args, const mp_obj_t *args) {
-    (void)n_args;
-
-    mp_obj_framebuf_t *self = MP_OBJ_TO_PTR(args[0]);
-    mp_int_t x = mp_obj_get_int(args[1]);
-    mp_int_t y = mp_obj_get_int(args[2]);
-    mp_int_t w = mp_obj_get_int(args[3]);
-    mp_int_t h = mp_obj_get_int(args[4]);
-    mp_int_t col = mp_obj_get_int(args[5]);
-
-    fill_rect(self, x, y, w, 1, col);
-    fill_rect(self, x, y + h - 1, w, 1, col);
-    fill_rect(self, x, y, 1, h, col);
-    fill_rect(self, x + w - 1, y, 1, h, col);
-
+STATIC mp_obj_t framebuf_rect(size_t n_args, const mp_obj_t *args_in) {
+    mp_obj_framebuf_t *self = MP_OBJ_TO_PTR(args_in[0]);
+    mp_int_t args[5]; // x, y, w, h, col
+    framebuf_args(args_in, args, 5);
+    if (n_args > 6 && mp_obj_is_true(args_in[6])) {
+        fill_rect(self, args[0], args[1], args[2], args[3], args[4]);
+    } else {
+        fill_rect(self, args[0], args[1], args[2], 1, args[4]);
+        fill_rect(self, args[0], args[1] + args[3] - 1, args[2], 1, args[4]);
+        fill_rect(self, args[0], args[1], 1, args[3], args[4]);
+        fill_rect(self, args[0] + args[2] - 1, args[1], 1, args[3], args[4]);
+    }
     return mp_const_none;
 }
-STATIC MP_DEFINE_CONST_FUN_OBJ_VAR_BETWEEN(framebuf_rect_obj, 6, 6, framebuf_rect);
-
-STATIC mp_obj_t framebuf_line(size_t n_args, const mp_obj_t *args) {
-    (void)n_args;
-
-    mp_obj_framebuf_t *self = MP_OBJ_TO_PTR(args[0]);
-    mp_int_t x1 = mp_obj_get_int(args[1]);
-    mp_int_t y1 = mp_obj_get_int(args[2]);
-    mp_int_t x2 = mp_obj_get_int(args[3]);
-    mp_int_t y2 = mp_obj_get_int(args[4]);
-    mp_int_t col = mp_obj_get_int(args[5]);
+STATIC MP_DEFINE_CONST_FUN_OBJ_VAR_BETWEEN(framebuf_rect_obj, 6, 7, framebuf_rect);
 
+STATIC void line(const mp_obj_framebuf_t *fb, mp_int_t x1, mp_int_t y1, mp_int_t x2, mp_int_t y2, mp_int_t col) {
     mp_int_t dx = x2 - x1;
     mp_int_t sx;
     if (dx > 0) {
@@ -452,12 +442,12 @@ STATIC mp_obj_t framebuf_line(size_t n_args, const mp_obj_t *args) {
     mp_int_t e = 2 * dy - dx;
     for (mp_int_t i = 0; i < dx; ++i) {
         if (steep) {
-            if (0 <= y1 && y1 < self->width && 0 <= x1 && x1 < self->height) {
-                setpixel(self, y1, x1, col);
+            if (0 <= y1 && y1 < fb->width && 0 <= x1 && x1 < fb->height) {
+                setpixel(fb, y1, x1, col);
             }
         } else {
-            if (0 <= x1 && x1 < self->width && 0 <= y1 && y1 < self->height) {
-                setpixel(self, x1, y1, col);
+            if (0 <= x1 && x1 < fb->width && 0 <= y1 && y1 < fb->height) {
+                setpixel(fb, x1, y1, col);
             }
         }
         while (e >= 0) {
@@ -468,31 +458,250 @@ STATIC mp_obj_t framebuf_line(size_t n_args, const mp_obj_t *args) {
         e += 2 * dy;
     }
 
-    if (0 <= x2 && x2 < self->width && 0 <= y2 && y2 < self->height) {
-        setpixel(self, x2, y2, col);
+    if (0 <= x2 && x2 < fb->width && 0 <= y2 && y2 < fb->height) {
+        setpixel(fb, x2, y2, col);
     }
+}
+
+STATIC mp_obj_t framebuf_line(size_t n_args, const mp_obj_t *args_in) {
+    (void)n_args;
+
+    mp_obj_framebuf_t *self = MP_OBJ_TO_PTR(args_in[0]);
+    mp_int_t args[5]; // x1, y1, x2, y2, col
+    framebuf_args(args_in, args, 5);
+
+    line(self, args[0], args[1], args[2], args[3], args[4]);
 
     return mp_const_none;
 }
 STATIC MP_DEFINE_CONST_FUN_OBJ_VAR_BETWEEN(framebuf_line_obj, 6, 6, framebuf_line);
 
-STATIC mp_obj_t framebuf_blit(size_t n_args, const mp_obj_t *args) {
-    mp_obj_framebuf_t *self = MP_OBJ_TO_PTR(args[0]);
-    mp_obj_t source_in = mp_obj_cast_to_native_base(args[1], MP_OBJ_FROM_PTR(&mp_type_framebuf));
+// Q2 Q1
+// Q3 Q4
+#define ELLIPSE_MASK_FILL (0x10)
+#define ELLIPSE_MASK_ALL (0x0f)
+#define ELLIPSE_MASK_Q1 (0x01)
+#define ELLIPSE_MASK_Q2 (0x02)
+#define ELLIPSE_MASK_Q3 (0x04)
+#define ELLIPSE_MASK_Q4 (0x08)
+
+STATIC void draw_ellipse_points(const mp_obj_framebuf_t *fb, mp_int_t cx, mp_int_t cy, mp_int_t x, mp_int_t y, mp_int_t col, mp_int_t mask) {
+    if (mask & ELLIPSE_MASK_FILL) {
+        if (mask & ELLIPSE_MASK_Q1) {
+            fill_rect(fb, cx, cy - y, x + 1, 1, col);
+        }
+        if (mask & ELLIPSE_MASK_Q2) {
+            fill_rect(fb, cx - x, cy - y, x + 1, 1, col);
+        }
+        if (mask & ELLIPSE_MASK_Q3) {
+            fill_rect(fb, cx - x, cy + y, x + 1, 1, col);
+        }
+        if (mask & ELLIPSE_MASK_Q4) {
+            fill_rect(fb, cx, cy + y, x + 1, 1, col);
+        }
+    } else {
+        setpixel_checked(fb, cx + x, cy - y, col, mask & ELLIPSE_MASK_Q1);
+        setpixel_checked(fb, cx - x, cy - y, col, mask & ELLIPSE_MASK_Q2);
+        setpixel_checked(fb, cx - x, cy + y, col, mask & ELLIPSE_MASK_Q3);
+        setpixel_checked(fb, cx + x, cy + y, col, mask & ELLIPSE_MASK_Q4);
+    }
+}
+
+STATIC mp_obj_t framebuf_ellipse(size_t n_args, const mp_obj_t *args_in) {
+    mp_obj_framebuf_t *self = MP_OBJ_TO_PTR(args_in[0]);
+    mp_int_t args[5];
+    framebuf_args(args_in, args, 5); // cx, cy, xradius, yradius, col
+    mp_int_t mask = (n_args > 6 && mp_obj_is_true(args_in[6])) ? ELLIPSE_MASK_FILL : 0;
+    if (n_args > 7) {
+        mask |= mp_obj_get_int(args_in[7]) & ELLIPSE_MASK_ALL;
+    } else {
+        mask |= ELLIPSE_MASK_ALL;
+    }
+    mp_int_t two_asquare = 2 * args[2] * args[2];
+    mp_int_t two_bsquare = 2 * args[3] * args[3];
+    mp_int_t x = args[2];
+    mp_int_t y = 0;
+    mp_int_t xchange = args[3] * args[3] * (1 - 2 * args[2]);
+    mp_int_t ychange = args[2] * args[2];
+    mp_int_t ellipse_error = 0;
+    mp_int_t stoppingx = two_bsquare * args[2];
+    mp_int_t stoppingy = 0;
+    while (stoppingx >= stoppingy) {   // 1st set of points,  y' > -1
+        draw_ellipse_points(self, args[0], args[1], x, y, args[4], mask);
+        y += 1;
+        stoppingy += two_asquare;
+        ellipse_error += ychange;
+        ychange += two_asquare;
+        if ((2 * ellipse_error + xchange) > 0) {
+            x -= 1;
+            stoppingx -= two_bsquare;
+            ellipse_error += xchange;
+            xchange += two_bsquare;
+        }
+    }
+    // 1st point set is done start the 2nd set of points
+    x = 0;
+    y = args[3];
+    xchange = args[3] * args[3];
+    ychange = args[2] * args[2] * (1 - 2 * args[3]);
+    ellipse_error = 0;
+    stoppingx = 0;
+    stoppingy = two_asquare * args[3];
+    while (stoppingx <= stoppingy) {  // 2nd set of points, y' < -1
+        draw_ellipse_points(self, args[0], args[1], x, y, args[4], mask);
+        x += 1;
+        stoppingx += two_bsquare;
+        ellipse_error += xchange;
+        xchange += two_bsquare;
+        if ((2 * ellipse_error + ychange) > 0) {
+            y -= 1;
+            stoppingy -= two_asquare;
+            ellipse_error += ychange;
+            ychange += two_asquare;
+        }
+    }
+    return mp_const_none;
+}
+STATIC MP_DEFINE_CONST_FUN_OBJ_VAR_BETWEEN(framebuf_ellipse_obj, 6, 8, framebuf_ellipse);
+
+#if MICROPY_PY_ARRAY && !MICROPY_ENABLE_DYNRUNTIME
+// TODO: poly needs mp_binary_get_size & mp_binary_get_val_array which aren't
+// available in dynruntime.h yet.
+
+STATIC mp_int_t poly_int(mp_buffer_info_t *bufinfo, size_t index) {
+    return mp_obj_get_int(mp_binary_get_val_array(bufinfo->typecode, bufinfo->buf, index));
+}
+
+STATIC mp_obj_t framebuf_poly(size_t n_args, const mp_obj_t *args_in) {
+    mp_obj_framebuf_t *self = MP_OBJ_TO_PTR(args_in[0]);
+
+    mp_int_t x = mp_obj_get_int(args_in[1]);
+    mp_int_t y = mp_obj_get_int(args_in[2]);
+
+    mp_buffer_info_t bufinfo;
+    mp_get_buffer_raise(args_in[3], &bufinfo, MP_BUFFER_READ);
+    // If an odd number of values was given, this rounds down to multiple of two.
+    int n_poly = bufinfo.len / (mp_binary_get_size('@', bufinfo.typecode, NULL) * 2);
+
+    if (n_poly == 0) {
+        return mp_const_none;
+    }
+
+    mp_int_t col = mp_obj_get_int(args_in[4]);
+    bool fill = n_args > 5 && mp_obj_is_true(args_in[5]);
+
+    if (fill) {
+        // This implements an integer version of http://alienryderflex.com/polygon_fill/
+
+        // The idea is for each scan line, compute the sorted list of x
+        // coordinates where the scan line intersects the polygon edges,
+        // then fill between each resulting pair.
+
+        // Restrict just to the scan lines that include the vertical extent of
+        // this polygon.
+        mp_int_t y_min = INT_MAX, y_max = INT_MIN;
+        for (int i = 0; i < n_poly; i++) {
+            mp_int_t py = poly_int(&bufinfo, i * 2 + 1);
+            y_min = MIN(y_min, py);
+            y_max = MAX(y_max, py);
+        }
+
+        for (mp_int_t row = y_min; row <= y_max; row++) {
+            // Each node is the x coordinate where an edge crosses this scan line.
+            mp_int_t nodes[n_poly];
+            int n_nodes = 0;
+            mp_int_t px1 = poly_int(&bufinfo, 0);
+            mp_int_t py1 = poly_int(&bufinfo, 1);
+            int i = n_poly * 2 - 1;
+            do {
+                mp_int_t py2 = poly_int(&bufinfo, i--);
+                mp_int_t px2 = poly_int(&bufinfo, i--);
+
+                // Don't include the bottom pixel of a given edge to avoid
+                // duplicating the node with the start of the next edge. This
+                // will miss some pixels on the boundary, and in particular
+                // at a local minima or inflection point.
+                if (py1 != py2 && ((py1 > row && py2 <= row) || (py1 <= row && py2 > row))) {
+                    mp_int_t node = (32 * px1 + 32 * (px2 - px1) * (row - py1) / (py2 - py1) + 16) / 32;
+                    nodes[n_nodes++] = node;
+                } else if (row == MAX(py1, py2)) {
+                    // At local-minima, try and manually fill in the pixels that get missed above.
+                    if (py1 < py2) {
+                        setpixel_checked(self, x + px2, y + py2, col, 1);
+                    } else if (py2 < py1) {
+                        setpixel_checked(self, x + px1, y + py1, col, 1);
+                    } else {
+                        // Even though this is a hline and would be faster to
+                        // use fill_rect, use line() because it handles x2 <
+                        // x1.
+                        line(self, x + px1, y + py1, x + px2, y + py2, col);
+                    }
+                }
+
+                px1 = px2;
+                py1 = py2;
+            } while (i >= 0);
+
+            if (!n_nodes) {
+                continue;
+            }
+
+            // Sort the nodes left-to-right (bubble-sort for code size).
+            i = 0;
+            while (i < n_nodes - 1) {
+                if (nodes[i] > nodes[i + 1]) {
+                    mp_int_t swap = nodes[i];
+                    nodes[i] = nodes[i + 1];
+                    nodes[i + 1] = swap;
+                    if (i) {
+                        i--;
+                    }
+                } else {
+                    i++;
+                }
+            }
+
+            // Fill between each pair of nodes.
+            for (i = 0; i < n_nodes; i += 2) {
+                fill_rect(self, x + nodes[i], y + row, (nodes[i + 1] - nodes[i]) + 1, 1, col);
+            }
+        }
+    } else {
+        // Outline only.
+        mp_int_t px1 = poly_int(&bufinfo, 0);
+        mp_int_t py1 = poly_int(&bufinfo, 1);
+        int i = n_poly * 2 - 1;
+        do {
+            mp_int_t py2 = poly_int(&bufinfo, i--);
+            mp_int_t px2 = poly_int(&bufinfo, i--);
+            line(self, x + px1, y + py1, x + px2, y + py2, col);
+            px1 = px2;
+            py1 = py2;
+        } while (i >= 0);
+    }
+
+    return mp_const_none;
+}
+STATIC MP_DEFINE_CONST_FUN_OBJ_VAR_BETWEEN(framebuf_poly_obj, 5, 6, framebuf_poly);
+#endif // MICROPY_PY_ARRAY && !MICROPY_ENABLE_DYNRUNTIME
+
+STATIC mp_obj_t framebuf_blit(size_t n_args, const mp_obj_t *args_in) {
+    mp_obj_framebuf_t *self = MP_OBJ_TO_PTR(args_in[0]);
+    mp_obj_t source_in = mp_obj_cast_to_native_base(args_in[1], MP_OBJ_FROM_PTR(&mp_type_framebuf));
     if (source_in == MP_OBJ_NULL) {
         mp_raise_TypeError(NULL);
     }
     mp_obj_framebuf_t *source = MP_OBJ_TO_PTR(source_in);
 
-    mp_int_t x = mp_obj_get_int(args[2]);
-    mp_int_t y = mp_obj_get_int(args[3]);
+    mp_int_t x = mp_obj_get_int(args_in[2]);
+    mp_int_t y = mp_obj_get_int(args_in[3]);
     mp_int_t key = -1;
     if (n_args > 4) {
-        key = mp_obj_get_int(args[4]);
+        key = mp_obj_get_int(args_in[4]);
     }
     mp_obj_framebuf_t *palette = NULL;
-    if (n_args > 5 && args[5] != mp_const_none) {
-        palette = MP_OBJ_TO_PTR(mp_obj_cast_to_native_base(args[5], MP_OBJ_FROM_PTR(&mp_type_framebuf)));
+    if (n_args > 5 && args_in[5] != mp_const_none) {
+        palette = MP_OBJ_TO_PTR(mp_obj_cast_to_native_base(args_in[5], MP_OBJ_FROM_PTR(&mp_type_framebuf)));
     }
 
     if (
@@ -539,19 +748,31 @@ STATIC mp_obj_t framebuf_scroll(mp_obj_t self_in, mp_obj_t xstep_in, mp_obj_t ys
     if (xstep < 0) {
         sx = 0;
         xend = self->width + xstep;
+        if (xend <= 0) {
+            return mp_const_none;
+        }
         dx = 1;
     } else {
         sx = self->width - 1;
         xend = xstep - 1;
+        if (xend >= sx) {
+            return mp_const_none;
+        }
         dx = -1;
     }
     if (ystep < 0) {
         y = 0;
         yend = self->height + ystep;
+        if (yend <= 0) {
+            return mp_const_none;
+        }
         dy = 1;
     } else {
         y = self->height - 1;
         yend = ystep - 1;
+        if (yend >= y) {
+            return mp_const_none;
+        }
         dy = -1;
     }
     for (; y != yend; y += dy) {
@@ -563,15 +784,15 @@ STATIC mp_obj_t framebuf_scroll(mp_obj_t self_in, mp_obj_t xstep_in, mp_obj_t ys
 }
 STATIC MP_DEFINE_CONST_FUN_OBJ_3(framebuf_scroll_obj, framebuf_scroll);
 
-STATIC mp_obj_t framebuf_text(size_t n_args, const mp_obj_t *args) {
+STATIC mp_obj_t framebuf_text(size_t n_args, const mp_obj_t *args_in) {
     // extract arguments
-    mp_obj_framebuf_t *self = MP_OBJ_TO_PTR(args[0]);
-    const char *str = mp_obj_str_get_str(args[1]);
-    mp_int_t x0 = mp_obj_get_int(args[2]);
-    mp_int_t y0 = mp_obj_get_int(args[3]);
+    mp_obj_framebuf_t *self = MP_OBJ_TO_PTR(args_in[0]);
+    const char *str = mp_obj_str_get_str(args_in[1]);
+    mp_int_t x0 = mp_obj_get_int(args_in[2]);
+    mp_int_t y0 = mp_obj_get_int(args_in[3]);
     mp_int_t col = 1;
     if (n_args >= 5) {
-        col = mp_obj_get_int(args[4]);
+        col = mp_obj_get_int(args_in[4]);
     }
 
     // loop over chars
@@ -610,34 +831,39 @@ STATIC const mp_rom_map_elem_t framebuf_locals_dict_table[] = {
     { MP_ROM_QSTR(MP_QSTR_vline), MP_ROM_PTR(&framebuf_vline_obj) },
     { MP_ROM_QSTR(MP_QSTR_rect), MP_ROM_PTR(&framebuf_rect_obj) },
     { MP_ROM_QSTR(MP_QSTR_line), MP_ROM_PTR(&framebuf_line_obj) },
+    { MP_ROM_QSTR(MP_QSTR_ellipse), MP_ROM_PTR(&framebuf_ellipse_obj) },
+    #if MICROPY_PY_ARRAY
+    { MP_ROM_QSTR(MP_QSTR_poly), MP_ROM_PTR(&framebuf_poly_obj) },
+    #endif
     { MP_ROM_QSTR(MP_QSTR_blit), MP_ROM_PTR(&framebuf_blit_obj) },
     { MP_ROM_QSTR(MP_QSTR_scroll), MP_ROM_PTR(&framebuf_scroll_obj) },
     { MP_ROM_QSTR(MP_QSTR_text), MP_ROM_PTR(&framebuf_text_obj) },
 };
 STATIC MP_DEFINE_CONST_DICT(framebuf_locals_dict, framebuf_locals_dict_table);
 
-STATIC const mp_obj_type_t mp_type_framebuf = {
-    { &mp_type_type },
-    .name = MP_QSTR_FrameBuffer,
-    .make_new = framebuf_make_new,
-    .buffer_p = { .get_buffer = framebuf_get_buffer },
-    .locals_dict = (mp_obj_dict_t *)&framebuf_locals_dict,
-};
+STATIC MP_DEFINE_CONST_OBJ_TYPE(
+    mp_type_framebuf,
+    MP_QSTR_FrameBuffer,
+    MP_TYPE_FLAG_NONE,
+    make_new, framebuf_make_new,
+    buffer, framebuf_get_buffer,
+    locals_dict, &framebuf_locals_dict
+    );
 #endif
 
 // this factory function is provided for backwards compatibility with old FrameBuffer1 class
-STATIC mp_obj_t legacy_framebuffer1(size_t n_args, const mp_obj_t *args) {
-    mp_obj_framebuf_t *o = mp_obj_malloc(mp_obj_framebuf_t, &mp_type_framebuf);
+STATIC mp_obj_t legacy_framebuffer1(size_t n_args, const mp_obj_t *args_in) {
+    mp_obj_framebuf_t *o = mp_obj_malloc(mp_obj_framebuf_t, (mp_obj_type_t *)&mp_type_framebuf);
 
     mp_buffer_info_t bufinfo;
-    mp_get_buffer_raise(args[0], &bufinfo, MP_BUFFER_WRITE);
+    mp_get_buffer_raise(args_in[0], &bufinfo, MP_BUFFER_WRITE);
     o->buf = bufinfo.buf;
 
-    o->width = mp_obj_get_int(args[1]);
-    o->height = mp_obj_get_int(args[2]);
+    o->width = mp_obj_get_int(args_in[1]);
+    o->height = mp_obj_get_int(args_in[2]);
     o->format = FRAMEBUF_MVLSB;
     if (n_args >= 4) {
-        o->stride = mp_obj_get_int(args[3]);
+        o->stride = mp_obj_get_int(args_in[3]);
     } else {
         o->stride = o->width;
     }
