@@ -1,5 +1,5 @@
 /*
- * Copyright 2019 NXP
+ * Copyright 2019 - 2021 NXP
  * All rights reserved.
  *
  * SPDX-License-Identifier: BSD-3-Clause
@@ -19,7 +19,7 @@ achieve better performance, it is depend on the IDE Floating point settings, if
 in IDE, clock_64b_t will switch to double type automatically. only support IAR and MDK here */
 #if __FPU_USED
 
-#if ((defined(__ICCARM__)) || (defined(__GNUC__)))
+#if (defined(__ICCARM__))
 
 #if (__ARMVFP__ >= __ARMFPV5__) && \
     (__ARM_FP == 0xE) /*0xe implies support for half, single and double precision operations*/
@@ -28,6 +28,14 @@ typedef double clock_64b_t;
 typedef uint64_t clock_64b_t;
 #endif
 
+#elif (defined(__GNUC__))
+
+#if (__ARM_FP == 0xE) /*0xe implies support for half, single and double precision operations*/
+typedef double clock_64b_t;
+#else
+typedef uint64_t clock_64b_t;
+#endif
+
 #elif defined(__CC_ARM) || defined(__ARMCC_VERSION)
 
 #if defined __TARGET_FPU_FPV5_D16
@@ -64,6 +72,26 @@ volatile uint32_t g_rtcXtalFreq;
  */
 static uint32_t CLOCK_GetPeriphClkFreq(void);
 
+/*!
+ * @brief Get the periph clock2 frequency.
+ *
+ * @return Periph clock2 frequency in Hz.
+ */
+static uint32_t CLOCK_GetPeriphClk2Freq(void);
+
+/*!
+ * @brief Get Flexspi selection frequency.
+ *
+ * @return Flexspi selection frequency in Hz.
+ */
+static uint32_t CLOCK_GetFlexspiSelFreq(void);
+
+/*!
+ * @brief Get the frequency of PLL USB1 software clock.
+ *
+ * @return The frequency of PLL USB1 software clock.
+ */
+static uint32_t CLOCK_GetPllUsb1SWFreq(void);
 /*******************************************************************************
  * Code
  ******************************************************************************/
@@ -73,7 +101,7 @@ static uint32_t CLOCK_GetPeriphClkFreq(void)
     uint32_t freq;
 
     /* Periph_clk2_clk ---> Periph_clk */
-    if (CCM->CBCDR & CCM_CBCDR_PERIPH_CLK_SEL_MASK)
+    if ((CCM->CBCDR & CCM_CBCDR_PERIPH_CLK_SEL_MASK) != 0UL)
     {
         switch (CCM->CBCMR & CCM_CBCMR_PERIPH_CLK2_SEL_MASK)
         {
@@ -131,6 +159,85 @@ static uint32_t CLOCK_GetPeriphClkFreq(void)
     return freq;
 }
 
+static uint32_t CLOCK_GetPllUsb1SWFreq(void)
+{
+    uint32_t freq;
+
+    switch ((CCM->CCSR & CCM_CCSR_PLL3_SW_CLK_SEL_MASK) >> CCM_CCSR_PLL3_SW_CLK_SEL_SHIFT)
+    {
+        case 0:
+        {
+            freq = CLOCK_GetPllFreq(kCLOCK_PllUsb1);
+            break;
+        }
+        case 1:
+        {
+            freq = 24000000UL;
+            break;
+        }
+        default:
+            freq = 0UL;
+            break;
+    }
+
+    return freq;
+}
+
+static uint32_t CLOCK_GetPeriphClk2Freq(void)
+{
+    uint32_t freq;
+
+    switch (CCM->CBCMR & CCM_CBCMR_PERIPH_CLK2_SEL_MASK)
+    {
+        /* Pll3_sw_clk ---> Periph_clk2_clk ---> Periph_clk */
+        case CCM_CBCMR_PERIPH_CLK2_SEL(0U):
+            freq = CLOCK_GetPllFreq(kCLOCK_PllUsb1);
+            break;
+
+        /* Osc_clk ---> Periph_clk2_clk ---> Periph_clk */
+        case CCM_CBCMR_PERIPH_CLK2_SEL(1U):
+            freq = CLOCK_GetOscFreq();
+            break;
+
+        case CCM_CBCMR_PERIPH_CLK2_SEL(2U):
+            freq = CLOCK_GetPllFreq(kCLOCK_PllSys);
+            break;
+
+        case CCM_CBCMR_PERIPH_CLK2_SEL(3U):
+        default:
+            freq = 0U;
+            break;
+    }
+
+    return freq;
+}
+
+static uint32_t CLOCK_GetFlexspiSelFreq(void)
+{
+    uint32_t freq;
+
+    switch ((CCM->CSCMR1 & CCM_CSCMR1_FLEXSPI_CLK_SEL_MASK) >> CCM_CSCMR1_FLEXSPI_CLK_SEL_SHIFT)
+    {
+        case 0:
+            freq = CLOCK_GetPllFreq(kCLOCK_PllSys);
+            break;
+        case 1:
+            freq = CLOCK_GetPllUsb1SWFreq();
+            break;
+        case 2:
+            freq = CLOCK_GetSysPfdFreq(kCLOCK_Pfd2);
+            break;
+        case 3:
+            freq = CLOCK_GetUsb1PfdFreq(kCLOCK_Pfd0);
+            break;
+        default:
+            freq = 0UL;
+            break;
+    }
+
+    return freq;
+}
+
 /*!
  * brief Initialize the external 24MHz clock.
  *
@@ -151,14 +258,14 @@ void CLOCK_InitExternalClk(bool bypassXtalOsc)
     assert(!bypassXtalOsc);
 
     CCM_ANALOG->MISC0_CLR = CCM_ANALOG_MISC0_XTAL_24M_PWD_MASK; /* Power up */
-    while ((XTALOSC24M->LOWPWR_CTRL & XTALOSC24M_LOWPWR_CTRL_XTALOSC_PWRUP_STAT_MASK) == 0)
+    while ((XTALOSC24M->LOWPWR_CTRL & XTALOSC24M_LOWPWR_CTRL_XTALOSC_PWRUP_STAT_MASK) == 0UL)
     {
     }
-    CCM_ANALOG->MISC0_SET = CCM_ANALOG_MISC0_OSC_XTALOK_EN_MASK; /* detect freq */
-    while ((CCM_ANALOG->MISC0 & CCM_ANALOG_MISC0_OSC_XTALOK_MASK) == 0)
+    CCM_ANALOG->MISC0_SET = (uint32_t)CCM_ANALOG_MISC0_OSC_XTALOK_EN_MASK; /* detect freq */
+    while ((CCM_ANALOG->MISC0 & CCM_ANALOG_MISC0_OSC_XTALOK_MASK) == 0UL)
     {
     }
-    CCM_ANALOG->MISC0_CLR = CCM_ANALOG_MISC0_OSC_XTALOK_EN_MASK;
+    CCM_ANALOG->MISC0_CLR = (uint32_t)CCM_ANALOG_MISC0_OSC_XTALOK_EN_MASK;
 }
 
 /*!
@@ -184,9 +291,13 @@ void CLOCK_DeinitExternalClk(void)
 void CLOCK_SwitchOsc(clock_osc_t osc)
 {
     if (osc == kCLOCK_RcOsc)
+    {
         XTALOSC24M->LOWPWR_CTRL_SET = XTALOSC24M_LOWPWR_CTRL_SET_OSC_SEL_MASK;
+    }
     else
+    {
         XTALOSC24M->LOWPWR_CTRL_CLR = XTALOSC24M_LOWPWR_CTRL_CLR_OSC_SEL_MASK;
+    }
 }
 
 /*!
@@ -235,14 +346,14 @@ uint32_t CLOCK_GetPerClkFreq(void)
     uint32_t freq;
 
     /* Osc_clk ---> PER Clock*/
-    if (CCM->CSCMR1 & CCM_CSCMR1_PERCLK_CLK_SEL_MASK)
+    if ((CCM->CSCMR1 & CCM_CSCMR1_PERCLK_CLK_SEL_MASK) != 0UL)
     {
         freq = CLOCK_GetOscFreq();
     }
     /* Periph_clk ---> AHB Clock ---> IPG Clock ---> PER Clock */
     else
     {
-        freq = CLOCK_GetFreq(kCLOCK_IpgClk);
+        freq = CLOCK_GetIpgFreq();
     }
 
     freq /= (((CCM->CSCMR1 & CCM_CSCMR1_PERCLK_PODF_MASK) >> CCM_CSCMR1_PERCLK_PODF_SHIFT) + 1U);
@@ -299,6 +410,15 @@ uint32_t CLOCK_GetFreq(clock_name_t name)
         case kCLOCK_Usb1PllPfd3Clk:
             freq = CLOCK_GetUsb1PfdFreq(kCLOCK_Pfd3);
             break;
+        case kCLOCK_Usb1SwClk:
+            freq = CLOCK_GetPllUsb1SWFreq();
+            break;
+        case kCLOCK_Usb1Sw60MClk:
+            freq = CLOCK_GetPllUsb1SWFreq() / 8UL;
+            break;
+        case kCLOCK_Usb1Sw80MClk:
+            freq = CLOCK_GetPllUsb1SWFreq() / 6UL;
+            break;
         case kCLOCK_SysPllClk:
             freq = CLOCK_GetPllFreq(kCLOCK_PllSys);
             break;
@@ -328,6 +448,56 @@ uint32_t CLOCK_GetFreq(clock_name_t name)
     return freq;
 }
 
+/*!
+ * brief Gets the frequency of selected clock root.
+ *
+ * param clockRoot The clock root used to get the frequency, please refer to @ref clock_root_t.
+ * return The frequency of selected clock root.
+ */
+uint32_t CLOCK_GetClockRootFreq(clock_root_t clockRoot)
+{
+    static const clock_name_t clockRootSourceArray[][4]  = CLOCK_ROOT_SOUCE;
+    static const clock_mux_t clockRootMuxTupleArray[]    = CLOCK_ROOT_MUX_TUPLE;
+    static const clock_div_t clockRootDivTupleArray[][2] = CLOCK_ROOT_DIV_TUPLE;
+    uint32_t freq                                        = 0UL;
+    clock_mux_t clockRootMuxTuple                        = clockRootMuxTupleArray[(uint8_t)clockRoot];
+    clock_div_t clockRootPreDivTuple                     = clockRootDivTupleArray[(uint8_t)clockRoot][0];
+    clock_div_t clockRootPostDivTuple                    = clockRootDivTupleArray[(uint8_t)clockRoot][1];
+    uint32_t clockRootMuxValue = (CCM_TUPLE_REG(CCM, clockRootMuxTuple) & CCM_TUPLE_MASK(clockRootMuxTuple)) >>
+                                 CCM_TUPLE_SHIFT(clockRootMuxTuple);
+    clock_name_t clockSourceName;
+
+    clockSourceName = clockRootSourceArray[(uint8_t)clockRoot][clockRootMuxValue];
+
+    assert(clockSourceName != kCLOCK_NoneName);
+
+    if (clockSourceName == kCLOCK_PeriphClk2)
+    {
+        freq = CLOCK_GetPeriphClk2Freq();
+    }
+    else if (clockSourceName == kCLOCK_FlexspiSel)
+    {
+        freq = CLOCK_GetFlexspiSelFreq();
+    }
+    else
+    {
+        freq = CLOCK_GetFreq(clockSourceName);
+    }
+
+    if (clockRootPreDivTuple != kCLOCK_NonePreDiv)
+    {
+        freq /= ((CCM_TUPLE_REG(CCM, clockRootPreDivTuple) & CCM_TUPLE_MASK(clockRootPreDivTuple)) >>
+                 CCM_TUPLE_SHIFT(clockRootPreDivTuple)) +
+                1UL;
+    }
+
+    freq /= ((CCM_TUPLE_REG(CCM, clockRootPostDivTuple) & CCM_TUPLE_MASK(clockRootPostDivTuple)) >>
+             CCM_TUPLE_SHIFT(clockRootPostDivTuple)) +
+            1UL;
+
+    return freq;
+}
+
 /*! brief Enable USB HS clock.
  *
  * This function only enables the access to USB HS prepheral, upper layer
@@ -341,10 +511,12 @@ uint32_t CLOCK_GetFreq(clock_name_t name)
  */
 bool CLOCK_EnableUsbhs0Clock(clock_usb_src_t src, uint32_t freq)
 {
+    uint32_t i;
     CCM->CCGR6 |= CCM_CCGR6_CG0_MASK;
     USB->USBCMD |= USBHS_USBCMD_RST_MASK;
-    for (volatile uint32_t i = 0; i < 400000;
-         i++) /* Add a delay between RST and RS so make sure there is a DP pullup sequence*/
+
+    /* Add a delay between RST and RS so make sure there is a DP pullup sequence*/
+    for (i = 0; i < 400000UL; i++)
     {
         __ASM("nop");
     }
@@ -364,8 +536,8 @@ bool CLOCK_EnableUsbhs0Clock(clock_usb_src_t src, uint32_t freq)
  */
 bool CLOCK_EnableUsbhs0PhyPllClock(clock_usb_phy_src_t src, uint32_t freq)
 {
-    const clock_usb_pll_config_t g_ccmConfigUsbPll = {.loopDivider = 0U};
-    if (CCM_ANALOG->PLL_USB1 & CCM_ANALOG_PLL_USB1_ENABLE_MASK)
+    static const clock_usb_pll_config_t g_ccmConfigUsbPll = {.loopDivider = 0U};
+    if ((CCM_ANALOG->PLL_USB1 & CCM_ANALOG_PLL_USB1_ENABLE_MASK) != 0UL)
     {
         CCM_ANALOG->PLL_USB1 |= CCM_ANALOG_PLL_USB1_EN_USB_CLKS_MASK;
     }
@@ -418,7 +590,7 @@ void CLOCK_InitSysPll(const clock_sys_pll_config_t *config)
                              CCM_ANALOG_PLL_SYS_SS_ENABLE(config->ss_enable) |
                              CCM_ANALOG_PLL_SYS_SS_STOP(config->ss_stop);
 
-    while ((CCM_ANALOG->PLL_SYS & CCM_ANALOG_PLL_SYS_LOCK_MASK) == 0)
+    while ((CCM_ANALOG->PLL_SYS & CCM_ANALOG_PLL_SYS_LOCK_MASK) == 0UL)
     {
     }
 
@@ -451,7 +623,7 @@ void CLOCK_InitUsb1Pll(const clock_usb_pll_config_t *config)
                            CCM_ANALOG_PLL_USB1_ENABLE_MASK | CCM_ANALOG_PLL_USB1_POWER_MASK |
                            CCM_ANALOG_PLL_USB1_EN_USB_CLKS_MASK | CCM_ANALOG_PLL_USB1_DIV_SELECT(config->loopDivider);
 
-    while ((CCM_ANALOG->PLL_USB1 & CCM_ANALOG_PLL_USB1_LOCK_MASK) == 0)
+    while ((CCM_ANALOG->PLL_USB1 & CCM_ANALOG_PLL_USB1_LOCK_MASK) == 0UL)
     {
     }
 
@@ -538,7 +710,7 @@ void CLOCK_InitAudioPll(const clock_audio_pll_config_t *config)
 
     CCM_ANALOG->PLL_AUDIO = pllAudio;
 
-    while ((CCM_ANALOG->PLL_AUDIO & CCM_ANALOG_PLL_AUDIO_LOCK_MASK) == 0)
+    while ((CCM_ANALOG->PLL_AUDIO & CCM_ANALOG_PLL_AUDIO_LOCK_MASK) == 0UL)
     {
     }
 
@@ -576,7 +748,7 @@ void CLOCK_InitEnetPll(const clock_enet_pll_config_t *config)
     CCM_ANALOG->PLL_ENET = (CCM_ANALOG->PLL_ENET & (~CCM_ANALOG_PLL_ENET_POWERDOWN_MASK)) | enet_pll;
 
     /* Wait for stable */
-    while ((CCM_ANALOG->PLL_ENET & CCM_ANALOG_PLL_ENET_LOCK_MASK) == 0)
+    while ((CCM_ANALOG->PLL_ENET & CCM_ANALOG_PLL_ENET_LOCK_MASK) == 0UL)
     {
     }
 
@@ -627,10 +799,10 @@ uint32_t CLOCK_GetPllFreq(clock_pll_t pll)
     {
         case kCLOCK_PllSys:
             /* PLL output frequency = Fref * (DIV_SELECT + NUM/DENOM). */
-            freqTmp = ((clock_64b_t)freq * ((clock_64b_t)(CCM_ANALOG->PLL_SYS_NUM))) /
-                      ((clock_64b_t)(CCM_ANALOG->PLL_SYS_DENOM));
+            freqTmp = ((clock_64b_t)freq * ((clock_64b_t)(CCM_ANALOG->PLL_SYS_NUM)));
+            freqTmp /= ((clock_64b_t)(CCM_ANALOG->PLL_SYS_DENOM));
 
-            if (CCM_ANALOG->PLL_SYS & CCM_ANALOG_PLL_SYS_DIV_SELECT_MASK)
+            if ((CCM_ANALOG->PLL_SYS & CCM_ANALOG_PLL_SYS_DIV_SELECT_MASK) != 0UL)
             {
                 freq *= 22U;
             }
@@ -643,7 +815,7 @@ uint32_t CLOCK_GetPllFreq(clock_pll_t pll)
             break;
 
         case kCLOCK_PllUsb1:
-            freq = (freq * ((CCM_ANALOG->PLL_USB1 & CCM_ANALOG_PLL_USB1_DIV_SELECT_MASK) ? 22U : 20U));
+            freq = (freq * (((CCM_ANALOG->PLL_USB1 & CCM_ANALOG_PLL_USB1_DIV_SELECT_MASK) != 0UL) ? 22U : 20U));
             break;
 
         case kCLOCK_PllAudio:
@@ -651,8 +823,8 @@ uint32_t CLOCK_GetPllFreq(clock_pll_t pll)
             divSelect =
                 (CCM_ANALOG->PLL_AUDIO & CCM_ANALOG_PLL_AUDIO_DIV_SELECT_MASK) >> CCM_ANALOG_PLL_AUDIO_DIV_SELECT_SHIFT;
 
-            freqTmp = ((clock_64b_t)freq * ((clock_64b_t)(CCM_ANALOG->PLL_AUDIO_NUM))) /
-                      ((clock_64b_t)(CCM_ANALOG->PLL_AUDIO_DENOM));
+            freqTmp = ((clock_64b_t)freq * ((clock_64b_t)(CCM_ANALOG->PLL_AUDIO_NUM)));
+            freqTmp /= ((clock_64b_t)(CCM_ANALOG->PLL_AUDIO_DENOM));
 
             freq = freq * divSelect + (uint32_t)freqTmp;
 
@@ -683,6 +855,7 @@ uint32_t CLOCK_GetPllFreq(clock_pll_t pll)
                     break;
 
                 default:
+                    /* Add comment to avoid MISRA C-2012 rule 16.4 */
                     break;
             }
 
@@ -697,6 +870,7 @@ uint32_t CLOCK_GetPllFreq(clock_pll_t pll)
                     break;
 
                 default:
+                    /* Add comment to avoid MISRA C-2012 rule 16.4 */
                     break;
             }
             break;
@@ -730,13 +904,14 @@ void CLOCK_InitSysPfd(clock_pfd_t pfd, uint8_t pfdFrac)
     uint32_t pfd528;
 
     pfd528 = CCM_ANALOG->PFD_528 &
-             ~((CCM_ANALOG_PFD_528_PFD0_CLKGATE_MASK | CCM_ANALOG_PFD_528_PFD0_FRAC_MASK) << (8 * pfdIndex));
+             ~(((uint32_t)((uint32_t)CCM_ANALOG_PFD_528_PFD0_CLKGATE_MASK | (uint32_t)CCM_ANALOG_PFD_528_PFD0_FRAC_MASK)
+                << (8UL * pfdIndex)));
 
     /* Disable the clock output first. */
-    CCM_ANALOG->PFD_528 = pfd528 | (CCM_ANALOG_PFD_528_PFD0_CLKGATE_MASK << (8 * pfdIndex));
+    CCM_ANALOG->PFD_528 = pfd528 | ((uint32_t)CCM_ANALOG_PFD_528_PFD0_CLKGATE_MASK << (8UL * pfdIndex));
 
     /* Set the new value and enable output. */
-    CCM_ANALOG->PFD_528 = pfd528 | (CCM_ANALOG_PFD_528_PFD0_FRAC(pfdFrac) << (8 * pfdIndex));
+    CCM_ANALOG->PFD_528 = pfd528 | (CCM_ANALOG_PFD_528_PFD0_FRAC(pfdFrac) << (8UL * pfdIndex));
 }
 
 /*!
@@ -748,7 +923,20 @@ void CLOCK_InitSysPfd(clock_pfd_t pfd, uint8_t pfdFrac)
  */
 void CLOCK_DeinitSysPfd(clock_pfd_t pfd)
 {
-    CCM_ANALOG->PFD_528 |= CCM_ANALOG_PFD_528_PFD0_CLKGATE_MASK << (8 * pfd);
+    CCM_ANALOG->PFD_528 |= (uint32_t)CCM_ANALOG_PFD_528_PFD0_CLKGATE_MASK << (8UL * (uint32_t)pfd);
+}
+
+/*!
+ * brief Check if Sys PFD is enabled
+ *
+ * param pfd PFD control name
+ * return PFD bypass status.
+ *         - true: power on.
+ *         - false: power off.
+ */
+bool CLOCK_IsSysPfdEnabled(clock_pfd_t pfd)
+{
+    return ((CCM_ANALOG->PFD_528 & (uint32_t)CCM_ANALOG_PFD_528_PFD0_CLKGATE_MASK << (8UL * (uint32_t)pfd)) == 0U);
 }
 
 /*!
@@ -767,13 +955,14 @@ void CLOCK_InitUsb1Pfd(clock_pfd_t pfd, uint8_t pfdFrac)
     uint32_t pfd480;
 
     pfd480 = CCM_ANALOG->PFD_480 &
-             ~((CCM_ANALOG_PFD_480_PFD0_CLKGATE_MASK | CCM_ANALOG_PFD_480_PFD0_FRAC_MASK) << (8 * pfdIndex));
+             ~((uint32_t)((uint32_t)CCM_ANALOG_PFD_480_PFD0_CLKGATE_MASK | (uint32_t)CCM_ANALOG_PFD_480_PFD0_FRAC_MASK)
+               << (8UL * pfdIndex));
 
     /* Disable the clock output first. */
-    CCM_ANALOG->PFD_480 = pfd480 | (CCM_ANALOG_PFD_480_PFD0_CLKGATE_MASK << (8 * pfdIndex));
+    CCM_ANALOG->PFD_480 = pfd480 | ((uint32_t)CCM_ANALOG_PFD_480_PFD0_CLKGATE_MASK << (8UL * pfdIndex));
 
     /* Set the new value and enable output. */
-    CCM_ANALOG->PFD_480 = pfd480 | (CCM_ANALOG_PFD_480_PFD0_FRAC(pfdFrac) << (8 * pfdIndex));
+    CCM_ANALOG->PFD_480 = pfd480 | (CCM_ANALOG_PFD_480_PFD0_FRAC(pfdFrac) << (8UL * pfdIndex));
 }
 
 /*!
@@ -785,7 +974,20 @@ void CLOCK_InitUsb1Pfd(clock_pfd_t pfd, uint8_t pfdFrac)
  */
 void CLOCK_DeinitUsb1Pfd(clock_pfd_t pfd)
 {
-    CCM_ANALOG->PFD_480 |= CCM_ANALOG_PFD_480_PFD0_CLKGATE_MASK << (8 * pfd);
+    CCM_ANALOG->PFD_480 |= (uint32_t)CCM_ANALOG_PFD_480_PFD0_CLKGATE_MASK << (8UL * (uint32_t)pfd);
+}
+
+/*!
+ * brief Check if Usb1 PFD is enabled
+ *
+ * param pfd PFD control name.
+ * return PFD bypass status.
+ *         - true: power on.
+ *         - false: power off.
+ */
+bool CLOCK_IsUsb1PfdEnabled(clock_pfd_t pfd)
+{
+    return ((CCM_ANALOG->PFD_480 & (uint32_t)CCM_ANALOG_PFD_480_PFD0_CLKGATE_MASK << (8UL * (uint32_t)pfd)) == 0U);
 }
 
 /*!
@@ -865,3 +1067,162 @@ uint32_t CLOCK_GetUsb1PfdFreq(clock_pfd_t pfd)
 
     return freq;
 }
+
+/*!
+ * brief Set the clock source and the divider of the clock output1.
+ *
+ * param selection The clock source to be output, please refer to clock_output1_selection_t.
+ * param divider The divider of the output clock signal, please refer to clock_output_divider_t.
+ */
+void CLOCK_SetClockOutput1(clock_output1_selection_t selection, clock_output_divider_t divider)
+{
+    uint32_t tmp32;
+
+    tmp32 = CCM->CCOSR;
+    if (selection == kCLOCK_DisableClockOutput1)
+    {
+        tmp32 &= ~CCM_CCOSR_CLKO1_EN_MASK;
+    }
+    else
+    {
+        tmp32 |= CCM_CCOSR_CLKO1_EN_MASK;
+        tmp32 &= ~(CCM_CCOSR_CLKO1_SEL_MASK | CCM_CCOSR_CLKO1_DIV_MASK);
+        tmp32 |= CCM_CCOSR_CLKO1_SEL(selection) | CCM_CCOSR_CLKO1_DIV(divider);
+    }
+    CCM->CCOSR = tmp32;
+}
+
+/*!
+ * brief Set the clock source and the divider of the clock output2.
+ *
+ * param selection The clock source to be output, please refer to clock_output2_selection_t.
+ * param divider The divider of the output clock signal, please refer to clock_output_divider_t.
+ */
+void CLOCK_SetClockOutput2(clock_output2_selection_t selection, clock_output_divider_t divider)
+{
+    uint32_t tmp32;
+
+    tmp32 = CCM->CCOSR;
+    if (selection == kCLOCK_DisableClockOutput2)
+    {
+        tmp32 &= CCM_CCOSR_CLKO2_EN_MASK;
+    }
+    else
+    {
+        tmp32 |= CCM_CCOSR_CLKO2_EN_MASK;
+        tmp32 &= ~(CCM_CCOSR_CLKO2_SEL_MASK | CCM_CCOSR_CLKO2_DIV_MASK);
+        tmp32 |= CCM_CCOSR_CLKO2_SEL(selection) | CCM_CCOSR_CLKO2_DIV(divider);
+    }
+
+    CCM->CCOSR = tmp32;
+}
+
+/*!
+ * brief Get the frequency of clock output1 clock signal.
+ *
+ * return The frequency of clock output1 clock signal.
+ */
+uint32_t CLOCK_GetClockOutCLKO1Freq(void)
+{
+    uint32_t freq = 0U;
+    uint32_t tmp32;
+
+    tmp32 = CCM->CCOSR;
+
+    if ((tmp32 & CCM_CCOSR_CLKO1_EN_MASK) != 0UL)
+    {
+        switch ((tmp32 & CCM_CCOSR_CLKO1_SEL_MASK) >> CCM_CCOSR_CLKO1_SEL_SHIFT)
+        {
+            case (uint32_t)kCLOCK_OutputPllUsb1Sw:
+                freq = CLOCK_GetPllUsb1SWFreq() / 2UL;
+                break;
+            case (uint32_t)kCLOCK_OutputPllSys:
+                freq = CLOCK_GetPllFreq(kCLOCK_PllSys) / 2UL;
+                break;
+            case (uint32_t)kCLOCK_OutputPllENET:
+                freq = CLOCK_GetPllFreq(kCLOCK_PllEnet500M) / 2UL;
+                break;
+            case (uint32_t)kCLOCK_OutputCoreClk:
+                freq = CLOCK_GetCoreFreq();
+                break;
+            case (uint32_t)kCLOCK_OutputIpgClk:
+                freq = CLOCK_GetIpgFreq();
+                break;
+            case (uint32_t)kCLOCK_OutputPerClk:
+                freq = CLOCK_GetPerClkFreq();
+                break;
+            case (uint32_t)kCLOCK_OutputPll4MainClk:
+                freq = CLOCK_GetPllFreq(kCLOCK_PllAudio);
+                break;
+            default:
+                /* This branch should never be hit. */
+                break;
+        }
+
+        freq /= (((tmp32 & CCM_CCOSR_CLKO1_DIV_MASK) >> CCM_CCOSR_CLKO1_DIV_SHIFT) + 1U);
+    }
+    else
+    {
+        freq = 0UL;
+    }
+
+    return freq;
+}
+
+/*!
+ * brief Get the frequency of clock output2 clock signal.
+ *
+ * return The frequency of clock output2 clock signal.
+ */
+uint32_t CLOCK_GetClockOutClkO2Freq(void)
+{
+    uint32_t freq = 0U;
+    uint32_t tmp32;
+
+    tmp32 = CCM->CCOSR;
+
+    if ((tmp32 & CCM_CCOSR_CLKO2_EN_MASK) != 0UL)
+    {
+        switch ((tmp32 & CCM_CCOSR_CLKO2_SEL_MASK) >> CCM_CCOSR_CLKO2_SEL_SHIFT)
+        {
+            case (uint32_t)kCLOCK_OutputLpi2cClk:
+                freq = CLOCK_GetClockRootFreq(kCLOCK_Lpi2cClkRoot);
+                break;
+            case (uint32_t)kCLOCK_OutputOscClk:
+                freq = CLOCK_GetOscFreq();
+                break;
+            case (uint32_t)kCLOCK_OutputLpspiClk:
+                freq = CLOCK_GetClockRootFreq(kCLOCK_LpspiClkRoot);
+                break;
+            case (uint32_t)kCLOCK_OutputSai1Clk:
+                freq = CLOCK_GetClockRootFreq(kCLOCK_Sai1ClkRoot);
+                break;
+            case (uint32_t)kCLOCK_OutputSai3Clk:
+                freq = CLOCK_GetClockRootFreq(kCLOCK_Sai3ClkRoot);
+                break;
+            case (uint32_t)kCLOCK_OutputTraceClk:
+                freq = CLOCK_GetClockRootFreq(kCLOCK_TraceClkRoot);
+                break;
+            case (uint32_t)kCLOCK_OutputFlexspiClk:
+                freq = CLOCK_GetClockRootFreq(kCLOCK_FlexspiClkRoot);
+                break;
+            case (uint32_t)kCLOCK_OutputUartClk:
+                freq = CLOCK_GetClockRootFreq(kCLOCK_UartClkRoot);
+                break;
+            case (uint32_t)kCLOCK_OutputSpdif0Clk:
+                freq = CLOCK_GetClockRootFreq(kCLOCK_SpdifClkRoot);
+                break;
+            default:
+                /* This branch should never be hit. */
+                break;
+        }
+
+        freq /= (((tmp32 & CCM_CCOSR_CLKO2_DIV_MASK) >> CCM_CCOSR_CLKO2_DIV_SHIFT) + 1U);
+    }
+    else
+    {
+        freq = 0UL;
+    }
+
+    return freq;
+}
