@@ -48,11 +48,7 @@
 // and carrier output.
 
 // Last available RMT channel that can transmit.
-#if ESP_IDF_VERSION < ESP_IDF_VERSION_VAL(4, 4, 0)
-#define RMT_LAST_TX_CHANNEL (RMT_CHANNEL_MAX - 1)
-#else
 #define RMT_LAST_TX_CHANNEL (SOC_RMT_TX_CANDIDATES_PER_GROUP - 1)
-#endif
 
 // Forward declaration
 extern const mp_obj_type_t esp32_rmt_type;
@@ -326,13 +322,13 @@ STATIC mp_obj_t esp32_rmt_write_pulses(size_t n_args, const mp_obj_t *args) {
         check_esp_err(rmt_wait_tx_done(self->channel_id, portMAX_DELAY));
     }
 
-    check_esp_err(rmt_write_items(self->channel_id, self->items, num_items, false));
-
     if (self->loop_en) {
         check_esp_err(rmt_set_tx_intr_en(self->channel_id, false));
         check_esp_err(rmt_set_tx_loop_mode(self->channel_id, true));
     }
 
+    check_esp_err(rmt_write_items(self->channel_id, self->items, num_items, false));
+
     return mp_const_none;
 }
 STATIC MP_DEFINE_CONST_FUN_OBJ_VAR_BETWEEN(esp32_rmt_write_pulses_obj, 2, 3, esp32_rmt_write_pulses);
@@ -372,10 +368,11 @@ STATIC const mp_rom_map_elem_t esp32_rmt_locals_dict_table[] = {
 };
 STATIC MP_DEFINE_CONST_DICT(esp32_rmt_locals_dict, esp32_rmt_locals_dict_table);
 
-const mp_obj_type_t esp32_rmt_type = {
-    { &mp_type_type },
-    .name = MP_QSTR_RMT,
-    .print = esp32_rmt_print,
-    .make_new = esp32_rmt_make_new,
-    .locals_dict = (mp_obj_dict_t *)&esp32_rmt_locals_dict,
-};
+MP_DEFINE_CONST_OBJ_TYPE(
+    esp32_rmt_type,
+    MP_QSTR_RMT,
+    MP_TYPE_FLAG_NONE,
+    make_new, esp32_rmt_make_new,
+    print, esp32_rmt_print,
+    locals_dict, &esp32_rmt_locals_dict
+    );
