@@ -35,6 +35,8 @@
 #include "modmachine.h"
 
 #include "driver/spi_master.h"
+#include "soc/gpio_sig_map.h"
+#include "soc/spi_pins.h"
 
 // SPI mappings by device, naming used by IDF old/new
 // upython   | ESP32     | ESP32S2   | ESP32S3 | ESP32C3
@@ -42,34 +44,30 @@
 // SPI(id=1) | HSPI/SPI2 | FSPI/SPI2 | SPI2    | SPI2
 // SPI(id=2) | VSPI/SPI3 | HSPI/SPI3 | SPI3    | err
 
+// Number of available hardware SPI peripherals.
+#if SOC_SPI_PERIPH_NUM > 2
+#define MICROPY_HW_SPI_MAX (2)
+#else
+#define MICROPY_HW_SPI_MAX (1)
+#endif
+
 // Default pins for SPI(id=1) aka IDF SPI2, can be overridden by a board
 #ifndef MICROPY_HW_SPI1_SCK
-#ifdef SPI2_IOMUX_PIN_NUM_CLK
 // Use IO_MUX pins by default.
 // If SPI lines are routed to other pins through GPIO matrix
 // routing adds some delay and lower limit applies to SPI clk freq
-#define MICROPY_HW_SPI1_SCK SPI2_IOMUX_PIN_NUM_CLK      // pin 14 on ESP32
-#define MICROPY_HW_SPI1_MOSI SPI2_IOMUX_PIN_NUM_MOSI    // pin 13 on ESP32
-#define MICROPY_HW_SPI1_MISO SPI2_IOMUX_PIN_NUM_MISO    // pin 12 on ESP32
-// Only for compatibility with IDF 4.2 and older
-#elif CONFIG_IDF_TARGET_ESP32S2
-#define MICROPY_HW_SPI1_SCK FSPI_IOMUX_PIN_NUM_CLK
-#define MICROPY_HW_SPI1_MOSI FSPI_IOMUX_PIN_NUM_MOSI
-#define MICROPY_HW_SPI1_MISO FSPI_IOMUX_PIN_NUM_MISO
-#else
-#define MICROPY_HW_SPI1_SCK HSPI_IOMUX_PIN_NUM_CLK
-#define MICROPY_HW_SPI1_MOSI HSPI_IOMUX_PIN_NUM_MOSI
-#define MICROPY_HW_SPI1_MISO HSPI_IOMUX_PIN_NUM_MISO
-#endif
+#define MICROPY_HW_SPI1_SCK SPI2_IOMUX_PIN_NUM_CLK
+#define MICROPY_HW_SPI1_MOSI SPI2_IOMUX_PIN_NUM_MOSI
+#define MICROPY_HW_SPI1_MISO SPI2_IOMUX_PIN_NUM_MISO
 #endif
 
 // Default pins for SPI(id=2) aka IDF SPI3, can be overridden by a board
 #ifndef MICROPY_HW_SPI2_SCK
 #if CONFIG_IDF_TARGET_ESP32
 // ESP32 has IO_MUX pins for VSPI/SPI3 lines, use them as defaults
-#define MICROPY_HW_SPI2_SCK VSPI_IOMUX_PIN_NUM_CLK      // pin 18
-#define MICROPY_HW_SPI2_MOSI VSPI_IOMUX_PIN_NUM_MOSI    // pin 23
-#define MICROPY_HW_SPI2_MISO VSPI_IOMUX_PIN_NUM_MISO    // pin 19
+#define MICROPY_HW_SPI2_SCK SPI3_IOMUX_PIN_NUM_CLK      // pin 18
+#define MICROPY_HW_SPI2_MOSI SPI3_IOMUX_PIN_NUM_MOSI    // pin 23
+#define MICROPY_HW_SPI2_MISO SPI3_IOMUX_PIN_NUM_MISO    // pin 19
 #elif CONFIG_IDF_TARGET_ESP32S2 || CONFIG_IDF_TARGET_ESP32S3
 // ESP32S2 and S3 uses GPIO matrix for SPI3 pins, no IO_MUX possible
 // Set defaults to the pins used by SPI2 in Octal mode
@@ -82,13 +80,6 @@
 #define MP_HW_SPI_MAX_XFER_BYTES (4092)
 #define MP_HW_SPI_MAX_XFER_BITS (MP_HW_SPI_MAX_XFER_BYTES * 8) // Has to be an even multiple of 8
 
-#if CONFIG_IDF_TARGET_ESP32C3
-#define HSPI_HOST SPI2_HOST
-#elif CONFIG_IDF_TARGET_ESP32S3
-#define HSPI_HOST SPI3_HOST
-#define FSPI_HOST SPI2_HOST
-#endif
-
 typedef struct _machine_hw_spi_default_pins_t {
     int8_t sck;
     int8_t mosi;
@@ -115,15 +106,15 @@ typedef struct _machine_hw_spi_obj_t {
 } machine_hw_spi_obj_t;
 
 // Default pin mappings for the hardware SPI instances
-STATIC const machine_hw_spi_default_pins_t machine_hw_spi_default_pins[2] = {
+STATIC const machine_hw_spi_default_pins_t machine_hw_spi_default_pins[MICROPY_HW_SPI_MAX] = {
     { .sck = MICROPY_HW_SPI1_SCK, .mosi = MICROPY_HW_SPI1_MOSI, .miso = MICROPY_HW_SPI1_MISO },
     #ifdef MICROPY_HW_SPI2_SCK
     { .sck = MICROPY_HW_SPI2_SCK, .mosi = MICROPY_HW_SPI2_MOSI, .miso = MICROPY_HW_SPI2_MISO },
     #endif
 };
 
-// Static objects mapping to HSPI and VSPI hardware peripherals
-STATIC machine_hw_spi_obj_t machine_hw_spi_obj[2];
+// Static objects mapping to SPI2 (and SPI3 if available) hardware peripherals.
+STATIC machine_hw_spi_obj_t machine_hw_spi_obj[MICROPY_HW_SPI_MAX];
 
 STATIC void machine_hw_spi_deinit_internal(machine_hw_spi_obj_t *self) {
     switch (spi_bus_remove_device(self->spi)) {
@@ -150,8 +141,8 @@ STATIC void machine_hw_spi_deinit_internal(machine_hw_spi_obj_t *self) {
 
     for (int i = 0; i < 3; i++) {
         if (pins[i] != -1) {
-            gpio_pad_select_gpio(pins[i]);
-            gpio_matrix_out(pins[i], SIG_GPIO_OUT_IDX, false, false);
+            esp_rom_gpio_pad_select_gpio(pins[i]);
+            esp_rom_gpio_connect_out_signal(pins[i], SIG_GPIO_OUT_IDX, false, false);
             gpio_set_direction(pins[i], GPIO_MODE_INPUT);
         }
     }
@@ -226,17 +217,6 @@ STATIC void machine_hw_spi_init_internal(
         changed = true;
     }
 
-    if (self->host != HSPI_HOST
-        #ifdef FSPI_HOST
-        && self->host != FSPI_HOST
-        #endif
-        #ifdef VSPI_HOST
-        && self->host != VSPI_HOST
-        #endif
-        ) {
-        mp_raise_msg_varg(&mp_type_ValueError, MP_ERROR_TEXT("SPI(%d) doesn't exist"), self->host);
-    }
-
     if (changed) {
         if (self->state == MACHINE_HW_SPI_STATE_INIT) {
             self->state = MACHINE_HW_SPI_STATE_DEINIT;
@@ -270,7 +250,7 @@ STATIC void machine_hw_spi_init_internal(
     #if CONFIG_IDF_TARGET_ESP32S2 || CONFIG_IDF_TARGET_ESP32S3 || CONFIG_IDF_TARGET_ESP32C3
     dma_chan = SPI_DMA_CH_AUTO;
     #else
-    if (self->host == HSPI_HOST) {
+    if (self->host == SPI2_HOST) {
         dma_chan = 1;
     } else {
         dma_chan = 2;
@@ -483,13 +463,14 @@ mp_obj_t machine_hw_spi_make_new(const mp_obj_type_t *type, size_t n_args, size_
 
     machine_hw_spi_obj_t *self;
     const machine_hw_spi_default_pins_t *default_pins;
-    if (args[ARG_id].u_int == 1) { // SPI2_HOST which is FSPI_HOST on ESP32Sx, HSPI_HOST on others
-        self = &machine_hw_spi_obj[0];
-        default_pins = &machine_hw_spi_default_pins[0];
+    mp_int_t spi_id = args[ARG_id].u_int;
+    if (1 <= spi_id && spi_id <= MICROPY_HW_SPI_MAX) {
+        self = &machine_hw_spi_obj[spi_id - 1];
+        default_pins = &machine_hw_spi_default_pins[spi_id - 1];
     } else {
-        self = &machine_hw_spi_obj[1];
-        default_pins = &machine_hw_spi_default_pins[1];
+        mp_raise_msg_varg(&mp_type_ValueError, MP_ERROR_TEXT("SPI(%d) doesn't exist"), spi_id);
     }
+
     self->base.type = &machine_spi_type;
 
     int8_t sck, mosi, miso;
