@@ -46,10 +46,8 @@
 // See https://github.com/micropython/micropython/issues/5489 for history
 #if CONFIG_FREERTOS_UNICORE
 #define MP_TASK_COREID (0)
-#elif ESP_IDF_VERSION >= ESP_IDF_VERSION_VAL(4, 2, 0)
-#define MP_TASK_COREID (1)
 #else
-#define MP_TASK_COREID (0)
+#define MP_TASK_COREID (1)
 #endif
 
 extern TaskHandle_t mp_main_task_handle;
@@ -57,7 +55,13 @@ extern TaskHandle_t mp_main_task_handle;
 extern ringbuf_t stdin_ringbuf;
 
 // Check the ESP-IDF error code and raise an OSError if it's not ESP_OK.
-void check_esp_err(esp_err_t code);
+#if MICROPY_ERROR_REPORTING <= MICROPY_ERROR_REPORTING_NORMAL
+#define check_esp_err(code) check_esp_err_(code)
+void check_esp_err_(esp_err_t code);
+#else
+#define check_esp_err(code) check_esp_err_(code, __FUNCTION__, __LINE__, __FILE__)
+void check_esp_err_(esp_err_t code, const char *func, const int line, const char *file);
+#endif
 
 uint32_t mp_hal_ticks_us(void);
 __attribute__((always_inline)) static inline uint32_t mp_hal_ticks_cpu(void) {
@@ -71,7 +75,7 @@ __attribute__((always_inline)) static inline uint32_t mp_hal_ticks_cpu(void) {
 }
 
 void mp_hal_delay_us(uint32_t);
-#define mp_hal_delay_us_fast(us) ets_delay_us(us)
+#define mp_hal_delay_us_fast(us) esp_rom_delay_us(us)
 void mp_hal_set_interrupt_char(int c);
 uint32_t mp_hal_get_cpu_freq(void);
 
@@ -88,18 +92,17 @@ void mp_hal_wake_main_task_from_isr(void);
 #define mp_hal_pin_obj_t gpio_num_t
 mp_hal_pin_obj_t machine_pin_get_id(mp_obj_t pin_in);
 #define mp_hal_get_pin_obj(o) machine_pin_get_id(o)
-#define mp_obj_get_pin(o) machine_pin_get_id(o) // legacy name; only to support esp8266/modonewire
 #define mp_hal_pin_name(p) (p)
 static inline void mp_hal_pin_input(mp_hal_pin_obj_t pin) {
-    gpio_pad_select_gpio(pin);
+    esp_rom_gpio_pad_select_gpio(pin);
     gpio_set_direction(pin, GPIO_MODE_INPUT);
 }
 static inline void mp_hal_pin_output(mp_hal_pin_obj_t pin) {
-    gpio_pad_select_gpio(pin);
+    esp_rom_gpio_pad_select_gpio(pin);
     gpio_set_direction(pin, GPIO_MODE_INPUT_OUTPUT);
 }
 static inline void mp_hal_pin_open_drain(mp_hal_pin_obj_t pin) {
-    gpio_pad_select_gpio(pin);
+    esp_rom_gpio_pad_select_gpio(pin);
     gpio_set_direction(pin, GPIO_MODE_INPUT_OUTPUT_OD);
 }
 static inline void mp_hal_pin_od_low(mp_hal_pin_obj_t pin) {
