@@ -45,6 +45,10 @@
 #include "gccollect.h"
 #include "user_interface.h"
 
+#if MICROPY_ESPNOW
+#include "modespnow.h"
+#endif
+
 STATIC char heap[38 * 1024];
 
 STATIC void mp_reset(void) {
@@ -70,9 +74,13 @@ STATIC void mp_reset(void) {
         args[1] = MP_OBJ_NEW_SMALL_INT(115200);
         args[0] = MP_OBJ_TYPE_GET_SLOT(&pyb_uart_type, make_new)(&pyb_uart_type, 2, 0, args);
         args[1] = MP_OBJ_NEW_SMALL_INT(1);
-        mp_uos_dupterm_obj.fun.var(2, args);
+        mp_os_dupterm_obj.fun.var(2, args);
     }
 
+    #if MICROPY_ESPNOW
+    espnow_deinit(mp_const_none);
+    #endif
+
     #if MICROPY_MODULE_FROZEN
     pyexec_frozen_module("_boot.py", false);
     pyexec_file_if_exists("boot.py");
