@@ -19,7 +19,6 @@
 #define MICROPY_OPT_MATH_FACTORIAL              (0)
 #define MICROPY_REPL_EMACS_KEYS                 (0)
 #define MICROPY_PY_BUILTINS_COMPLEX             (0)
-#define MICROPY_MODULE_ATTR_DELEGATION          (0)
 #define MICROPY_PY_FUNCTION_ATTRS               (0)
 #define MICROPY_PY_DELATTR_SETATTR              (0)
 #define MICROPY_PY_BUILTINS_STR_CENTER          (0)
@@ -36,8 +35,8 @@
 #define MICROPY_PY_MATH_FACTORIAL               (0)
 #define MICROPY_PY_MATH_ISCLOSE                 (0)
 #define MICROPY_PY_SYS_PS1_PS2                  (0)
-#define MICROPY_PY_UBINASCII_CRC32              (0)
-#define MICROPY_PY_URANDOM_EXTRA_FUNCS          (0)
+#define MICROPY_PY_BINASCII_CRC32               (0)
+#define MICROPY_PY_RANDOM_EXTRA_FUNCS           (0)
 
 // Configure other options.
 #define MICROPY_OBJ_REPR            (MICROPY_OBJ_REPR_C)
@@ -54,9 +53,11 @@
 #define MICROPY_REPL_EVENT_DRIVEN   (0)
 #define MICROPY_USE_INTERNAL_ERRNO  (1)
 #define MICROPY_PY_BUILTINS_HELP_TEXT esp_help_text
-#define MICROPY_PY_UHASHLIB_SHA1    (MICROPY_PY_USSL && MICROPY_SSL_AXTLS)
-#define MICROPY_PY_URANDOM_SEED_INIT_FUNC (*WDEV_HWRNG)
-#define MICROPY_PY_UTIME_MP_HAL     (1)
+#define MICROPY_PY_HASHLIB_SHA1     (MICROPY_PY_SSL && MICROPY_SSL_AXTLS)
+#define MICROPY_PY_RANDOM_SEED_INIT_FUNC (*WDEV_HWRNG)
+#define MICROPY_PY_TIME_GMTIME_LOCALTIME_MKTIME (1)
+#define MICROPY_PY_TIME_TIME_TIME_NS (1)
+#define MICROPY_PY_TIME_INCLUDEFILE "ports/esp8266/modtime.c"
 #define MICROPY_PY_LWIP             (1)
 #define MICROPY_PY_LWIP_SOCK_RAW    (1)
 #define MICROPY_PY_MACHINE          (1)
@@ -64,7 +65,6 @@
 #define MICROPY_PY_MACHINE_BITSTREAM (1)
 #define MICROPY_PY_MACHINE_PULSE    (1)
 #define MICROPY_PY_MACHINE_PWM      (1)
-#define MICROPY_PY_MACHINE_PWM_INIT (1)
 #define MICROPY_PY_MACHINE_PWM_DUTY (1)
 #define MICROPY_PY_MACHINE_PWM_INCLUDEFILE "ports/esp8266/machine_pwm.c"
 #define MICROPY_PY_MACHINE_I2C      (1)
@@ -77,18 +77,19 @@
 #endif
 #define MICROPY_PY_NETWORK_INCLUDEFILE "ports/esp8266/modnetwork.h"
 #define MICROPY_PY_NETWORK_MODULE_GLOBALS_INCLUDEFILE "ports/esp8266/modnetwork_globals.h"
-#define MICROPY_PY_UWEBSOCKET       (1)
+#define MICROPY_PY_WEBSOCKET        (1)
 #define MICROPY_PY_ONEWIRE          (1)
 #define MICROPY_PY_WEBREPL          (1)
 #define MICROPY_PY_WEBREPL_DELAY    (20)
 #define MICROPY_PY_WEBREPL_STATIC_FILEBUF (1)
-#define MICROPY_PY_UOS_INCLUDEFILE  "ports/esp8266/moduos.c"
+#define MICROPY_PY_OS_INCLUDEFILE   "ports/esp8266/modos.c"
 #define MICROPY_PY_OS_DUPTERM       (2)
-#define MICROPY_PY_UOS_DUPTERM_NOTIFY (1)
-#define MICROPY_PY_UOS_DUPTERM_STREAM_DETACHED_ATTACHED (1)
-#define MICROPY_PY_UOS_UNAME        (1)
-#define MICROPY_PY_UOS_UNAME_RELEASE_DYNAMIC (1)
-#define MICROPY_PY_UOS_URANDOM      (1)
+#define MICROPY_PY_OS_DUPTERM_NOTIFY (1)
+#define MICROPY_PY_OS_DUPTERM_STREAM_DETACHED_ATTACHED (1)
+#define MICROPY_PY_OS_SYNC          (1)
+#define MICROPY_PY_OS_UNAME         (1)
+#define MICROPY_PY_OS_UNAME_RELEASE_DYNAMIC (1)
+#define MICROPY_PY_OS_URANDOM       (1)
 #define MICROPY_LONGINT_IMPL        (MICROPY_LONGINT_IMPL_MPZ)
 #define MICROPY_FLOAT_IMPL          (MICROPY_FLOAT_IMPL_FLOAT)
 #define MICROPY_WARNINGS            (1)
