@@ -33,6 +33,7 @@
 #include "shared/readline/readline.h"
 #include "shared/runtime/gchelper.h"
 #include "shared/runtime/pyexec.h"
+#include "shared/runtime/softtimer.h"
 #include "ticks.h"
 #include "tusb.h"
 #include "led.h"
@@ -42,7 +43,16 @@
 #if MICROPY_PY_LWIP
 #include "lwip/init.h"
 #include "lwip/apps/mdns.h"
+#if MICROPY_PY_NETWORK_CYW43
+#include "lib/cyw43-driver/src/cyw43.h"
 #endif
+#endif
+
+#if MICROPY_PY_BLUETOOTH
+#include "mpbthciport.h"
+#include "extmod/modbluetooth.h"
+#endif
+
 #include "systick.h"
 #include "extmod/modnetwork.h"
 
@@ -54,12 +64,8 @@ int main(void) {
     board_init();
     ticks_init();
     tusb_init();
-    led_init();
     pendsv_init();
 
-    mp_stack_set_top(&_estack);
-    mp_stack_set_limit(&_estack - &_sstack - 1024);
-
     #if MICROPY_PY_LWIP
     // lwIP doesn't allow to reinitialise itself by subsequent calls to this function
     // because the system timeout list (next_timeout) is only ever reset by BSS clearing.
@@ -68,10 +74,32 @@ int main(void) {
     #if LWIP_MDNS_RESPONDER
     mdns_resp_init();
     #endif
+
     systick_enable_dispatch(SYSTICK_DISPATCH_LWIP, mod_network_lwip_poll_wrapper);
     #endif
+    #if MICROPY_PY_BLUETOOTH
+    mp_bluetooth_hci_init();
+    #endif
+
+    #if MICROPY_PY_NETWORK_CYW43
+    {
+        cyw43_init(&cyw43_state);
+        uint8_t buf[8];
+        memcpy(&buf[0], "PYBD", 4);
+        mp_hal_get_mac_ascii(MP_HAL_MAC_WLAN0, 8, 4, (char *)&buf[4]);
+        cyw43_wifi_ap_set_ssid(&cyw43_state, 8, buf);
+        cyw43_wifi_ap_set_password(&cyw43_state, 8, (const uint8_t *)"pybd0123");
+    }
+    #endif
 
     for (;;) {
+        #if defined(MICROPY_HW_LED1)
+        led_init();
+        #endif
+
+        mp_stack_set_top(&_estack);
+        mp_stack_set_limit(&_estack - &_sstack - 1024);
+
         gc_init(&_gc_heap_start, &_gc_heap_end);
         mp_init();
 
@@ -83,7 +111,7 @@ int main(void) {
         readline_init0();
 
         // Execute _boot.py to set up the filesystem.
-        pyexec_frozen_module("_boot.py");
+        pyexec_frozen_module("_boot.py", false);
 
         // Execute user scripts.
         int ret = pyexec_file_if_exists("boot.py");
@@ -116,10 +144,15 @@ int main(void) {
         #if MICROPY_PY_MACHINE_I2S
         machine_i2s_deinit_all();
         #endif
+        #if MICROPY_PY_BLUETOOTH
+        mp_bluetooth_deinit();
+        #endif
         #if MICROPY_PY_NETWORK
         mod_network_deinit();
         #endif
+        machine_uart_deinit_all();
         machine_pwm_deinit_all();
+        soft_timer_deinit();
         gc_sweep_all();
         mp_deinit();
     }
@@ -145,50 +178,3 @@ void MP_WEAK __assert_func(const char *file, int line, const char *func, const c
     }
 }
 #endif
-
-const char mimxrt_help_text[] =
-    "Welcome to MicroPython!\n"
-    "\n"
-    "For online help please visit https://micropython.org/help/.\n"
-    "\n"
-    "For access to the hardware use the 'machine' module. \n"
-    "\n"
-    "Quick overview of some objects:\n"
-    "  machine.Pin(pin) -- get a pin, eg machine.Pin(0)\n"
-    "  machine.Pin(pin, m, [p]) -- get a pin and configure it for IO mode m, pull mode p\n"
-    "    methods: init(..), value([v]), high(), low())\n"
-    "\n"
-    "    Pins are numbered board specific, either 0-n, or 'D0'-'Dn', or 'A0' - 'An',\n"
-    "    according to the boards's pinout sheet.\n"
-    "    Pin IO modes are: Pin.IN, Pin.OUT, Pin.OPEN_DRAIN\n"
-    "    Pin pull modes are: Pin.PULL_UP, Pin.PULL_UP_47K, Pin.PULL_UP_22K, Pin.PULL_DOWN, Pin.PULL_HOLD\n"
-    "  machine.ADC(pin) -- make an analog object from a pin\n"
-    "    methods: read_u16()\n"
-    "  machine.UART(id, baudrate=115200) -- create an UART object (id=1 - 8, board-specific)\n"
-    "    methods: init(), write(buf), any()\n"
-    "             buf=read(n), readinto(buf), buf=readline()\n"
-    "    The RX and TX pins are fixed and board-specific.\n"
-    "  machine.SoftI2C() -- create a Soft I2C object\n"
-    "  machine.I2C(id) -- create a HW I2C object\n"
-    "    methods: readfrom(addr, buf, stop=True), writeto(addr, buf, stop=True)\n"
-    "             readfrom_mem(addr, memaddr, arg), writeto_mem(addr, memaddr, arg)\n"
-    "    SoftI2C allows to use any pin for sda and scl, HW I2C id's and pins are fixed\n"
-    "  machine.SoftSPI(baudrate=1000000) -- create a Soft SPI object\n"
-    "  machine.SPI(id, baudrate=1000000) -- create a HW SPI object\n"
-    "    methods: read(nbytes, write=0x00), write(buf), write_readinto(wr_buf, rd_buf)\n"
-    "    SoftSPI allows to use any pin for SPI, HW SPI id's and pins are fixed\n"
-    "  machine.Timer(id, freq, callback) -- create a hardware timer object (id=0,1,2)\n"
-    "    eg: machine.Timer(freq=1, callback=lambda t:print(t))\n"
-    "  machine.RTC() -- create a Real Time Clock object\n"
-    "    methods: init(), datetime([dateime_tuple]), now()\n"
-    "  machine.PWM(pin, freq, duty_u16[, kw_opts]) -- create a PWM object\n"
-    "    methods: init(), duty_u16([value]), duty_ns([value]), freq([value])\n"
-    "\n"
-    "Useful control commands:\n"
-    "  CTRL-C -- interrupt a running program\n"
-    "  CTRL-D -- on a blank line, do a soft reset of the board\n"
-    "  CTRL-E -- on a blank line, enter paste mode\n"
-    "\n"
-    "For further help on a specific object, type help(obj)\n"
-    "For a list of available modules, type help('modules')\n"
-;
