@@ -36,7 +36,7 @@ extern uint8_t __StackTop, __StackBottom;
 
 void *core_state[2];
 
-// This will be non-NULL while Python code is execting.
+// This will be non-NULL while Python code is executing.
 STATIC void *(*core1_entry)(void *) = NULL;
 
 STATIC void *core1_arg = NULL;
@@ -116,7 +116,13 @@ STATIC void core1_entry_wrapper(void) {
     // returning from here will loop the core forever (WFI)
 }
 
-void mp_thread_create(void *(*entry)(void *), void *arg, size_t *stack_size) {
+mp_uint_t mp_thread_get_id(void) {
+    // On RP2, there are only two threads, one for each core, so the thread id
+    // is the core number.
+    return get_core_num();
+}
+
+mp_uint_t mp_thread_create(void *(*entry)(void *), void *arg, size_t *stack_size) {
     // Check if core1 is already in use.
     if (core1_entry != NULL) {
         mp_raise_msg(&mp_type_OSError, MP_ERROR_TEXT("core1 in use"));
@@ -144,6 +150,8 @@ void mp_thread_create(void *(*entry)(void *), void *arg, size_t *stack_size) {
 
     // Adjust stack_size to provide room to recover from hitting the limit.
     *stack_size -= 512;
+
+    return 1;
 }
 
 void mp_thread_start(void) {
