@@ -28,7 +28,7 @@
 #include "py/mphal.h"
 #include "adc.h"
 
-#if defined(STM32F0) || defined(STM32G0) || defined(STM32G4) || defined(STM32H7) || defined(STM32L0) || defined(STM32L4) || defined(STM32WB) || defined(STM32WL)
+#if defined(STM32F0) || defined(STM32G0) || defined(STM32G4) || defined(STM32H5) || defined(STM32H7) || defined(STM32L0) || defined(STM32L4) || defined(STM32WB) || defined(STM32WL)
 #define ADC_V2 (1)
 #else
 #define ADC_V2 (0)
@@ -46,7 +46,11 @@
 #define ADC_STAB_DELAY_US (1)
 #define ADC_TEMPSENSOR_DELAY_US (10)
 #elif defined(STM32G4)
-#define ADC_STAB_DELAY_US (1) // TODO: Check if this is enough
+#define ADC_STAB_DELAY_US (20)
+#elif defined(STM32H5)
+// Stabilization delay = 1 conversion cycle
+// ADC clk = PDIV / 16 = 250 MHz / 16 = 15.625 MHz -> 64 ns -> select 1 us
+#define ADC_STAB_DELAY_US (1)
 #elif defined(STM32L4)
 #define ADC_STAB_DELAY_US (10)
 #elif defined(STM32WB)
@@ -62,6 +66,11 @@
 #elif defined(STM32G4)
 #define ADC_SAMPLETIME_DEFAULT      ADC_SAMPLETIME_12CYCLES_5
 #define ADC_SAMPLETIME_DEFAULT_INT  ADC_SAMPLETIME_247CYCLES_5
+#elif defined(STM32H5)
+// Worst case sampling time: slow channel, 12 bits, 680 ohms -> 165 ns
+// ADC clk = PDIV / 16 = 250 MHz / 16 = 15.625 MHz -> 64 ns -> 2.57 cycles -> select 6.5 cycles
+#define ADC_SAMPLETIME_DEFAULT      ADC_SAMPLETIME_6CYCLES_5
+#define ADC_SAMPLETIME_DEFAULT_INT  ADC_SAMPLETIME_247CYCLES_5
 #elif defined(STM32H7)
 #define ADC_SAMPLETIME_DEFAULT      ADC_SAMPLETIME_8CYCLES_5
 #define ADC_SAMPLETIME_DEFAULT_INT  ADC_SAMPLETIME_387CYCLES_5
@@ -79,8 +88,65 @@
 // Timeout for waiting for end-of-conversion
 #define ADC_EOC_TIMEOUT_MS (10)
 
-// This is a synthesised channel representing the maximum ADC reading (useful to scale other channels)
-#define ADC_CHANNEL_VREF (0xffff)
+// Channel IDs for machine.ADC object
+typedef enum _machine_adc_internal_ch_t {
+    // Regular external ADC inputs (0..19)
+    MACHINE_ADC_EXT_CH_0 = 0,
+    MACHINE_ADC_EXT_CH_19 = 19,
+
+    // Internal ADC channels (256..)
+    MACHINE_ADC_INT_CH_VREFINT = 256,
+    MACHINE_ADC_INT_CH_TEMPSENSOR,
+    #if defined(ADC_CHANNEL_VBAT)
+    MACHINE_ADC_INT_CH_VBAT,
+    #endif
+    #if defined(ADC_CHANNEL_VDDCORE)
+    MACHINE_ADC_INT_CH_VDDCORE,
+    #endif
+
+    // This is a synthesised channel representing the maximum ADC reading (useful to scale other channels)
+    MACHINE_ADC_CH_VREF = 0xffff // 0xffff for backward compatibility
+} machine_adc_internal_ch_t;
+
+// Convert machine_adc_internal_ch_t value to STM32 library ADC channel literal.
+// This function is required as literals are uint32_t types that don't map with MP_ROM_INT (31 bit signed).
+STATIC uint32_t adc_ll_channel(uint32_t channel_id) {
+    uint32_t adc_ll_ch;
+    switch (channel_id) {
+        // external channels map 1:1
+        case MACHINE_ADC_EXT_CH_0 ... MACHINE_ADC_EXT_CH_19:
+            adc_ll_ch = channel_id;
+            break;
+
+        // internal channels are converted to STM32 ADC defines
+        case MACHINE_ADC_INT_CH_VREFINT:
+            adc_ll_ch = ADC_CHANNEL_VREFINT;
+            break;
+        case MACHINE_ADC_INT_CH_TEMPSENSOR:
+            #if defined(STM32G4)
+            adc_ll_ch = ADC_CHANNEL_TEMPSENSOR_ADC1;
+            #else
+            adc_ll_ch = ADC_CHANNEL_TEMPSENSOR;
+            #endif
+            break;
+        #if defined(ADC_CHANNEL_VBAT)
+        case MACHINE_ADC_INT_CH_VBAT:
+            adc_ll_ch = ADC_CHANNEL_VBAT;
+            break;
+        #endif
+        #if defined(ADC_CHANNEL_VDDCORE)
+        case MACHINE_ADC_INT_CH_VDDCORE:
+            adc_ll_ch = ADC_CHANNEL_VDDCORE;
+            break;
+        #endif
+
+        // To save code memory for costly error handling, default to Vref for unknown channels
+        default:
+            adc_ll_ch = ADC_CHANNEL_VREFINT;
+            break;
+    };
+    return adc_ll_ch;
+}
 
 static inline void adc_stabilisation_delay_us(uint32_t us) {
     mp_hal_delay_us(us + 1);
@@ -113,7 +179,9 @@ void adc_config(ADC_TypeDef *adc, uint32_t bits) {
     __HAL_RCC_ADC_CLK_ENABLE();
     #else
     if (adc == ADC1) {
-        #if defined(STM32G4) || defined(STM32H7)
+        #if defined(STM32H5)
+        __HAL_RCC_ADC_CLK_ENABLE();
+        #elif defined(STM32G4) || defined(STM32H7)
         __HAL_RCC_ADC12_CLK_ENABLE();
         #else
         __HAL_RCC_ADC1_CLK_ENABLE();
@@ -121,7 +189,9 @@ void adc_config(ADC_TypeDef *adc, uint32_t bits) {
     }
     #if defined(ADC2)
     if (adc == ADC2) {
-        #if defined(STM32G4) || defined(STM32H7)
+        #if defined(STM32H5)
+        __HAL_RCC_ADC_CLK_ENABLE();
+        #elif defined(STM32G4) || defined(STM32H7)
         __HAL_RCC_ADC12_CLK_ENABLE();
         #else
         __HAL_RCC_ADC2_CLK_ENABLE();
@@ -144,6 +214,8 @@ void adc_config(ADC_TypeDef *adc, uint32_t bits) {
     adc->CFGR2 = 2 << ADC_CFGR2_CKMODE_Pos; // PCLK/4 (synchronous clock mode)
     #elif defined(STM32F4) || defined(STM32F7) || defined(STM32L4)
     ADCx_COMMON->CCR = 0; // ADCPR=PCLK/2
+    #elif defined(STM32G4) || defined(STM32H5)
+    ADC12_COMMON->CCR = 7 << ADC_CCR_PRESC_Pos; // PCLK/16 (asynchronous clock mode)
     #elif defined(STM32H7A3xx) || defined(STM32H7A3xxQ) || defined(STM32H7B3xx) || defined(STM32H7B3xxQ)
     ADC12_COMMON->CCR = 3 << ADC_CCR_CKMODE_Pos;
     #elif defined(STM32H7)
@@ -157,18 +229,18 @@ void adc_config(ADC_TypeDef *adc, uint32_t bits) {
     ADC_COMMON->CCR = 0 << ADC_CCR_PRESC_Pos; // PRESC=1
     #endif
 
-    #if defined(STM32H7) || defined(STM32L4) || defined(STM32WB)
+    #if defined(STM32G4) || defined(STM32H5) || defined(STM32H7) || defined(STM32L4) || defined(STM32WB)
     if (adc->CR & ADC_CR_DEEPPWD) {
         adc->CR = 0; // disable deep powerdown
     }
     #endif
 
-    #if defined(STM32H7) || defined(STM32L0) || defined(STM32L4) || defined(STM32WB) || defined(STM32WL)
+    #if defined(STM32G4) || defined(STM32H5) || defined(STM32H7) || defined(STM32L0) || defined(STM32L4) || defined(STM32WB) || defined(STM32WL)
     if (!(adc->CR & ADC_CR_ADVREGEN)) {
         adc->CR = ADC_CR_ADVREGEN; // enable VREG
         #if defined(STM32H7)
         mp_hal_delay_us(10); // T_ADCVREG_STUP
-        #elif defined(STM32L4) || defined(STM32WB)
+        #elif defined(STM32G4) || defined(STM32L4) || defined(STM32WB)
         mp_hal_delay_us(20); // T_ADCVREG_STUP
         #endif
     }
@@ -179,7 +251,7 @@ void adc_config(ADC_TypeDef *adc, uint32_t bits) {
         // ADC isn't enabled so calibrate it now
         #if defined(STM32F0) || defined(STM32G0) || defined(STM32L0) || defined(STM32WL)
         LL_ADC_StartCalibration(adc);
-        #elif defined(STM32G4) || defined(STM32L4) || defined(STM32WB)
+        #elif defined(STM32G4) || defined(STM32H5) || defined(STM32L4) || defined(STM32WB)
         LL_ADC_StartCalibration(adc, LL_ADC_SINGLE_ENDED);
         #else
         LL_ADC_StartCalibration(adc, LL_ADC_CALIB_OFFSET_LINEARITY, LL_ADC_SINGLE_ENDED);
@@ -225,7 +297,7 @@ void adc_config(ADC_TypeDef *adc, uint32_t bits) {
     adc->CR2 = (adc->CR2 & ~cr2_clr) | cr2;
     adc->SQR1 = 1 << ADC_SQR1_L_Pos; // 1 conversion in regular sequence
 
-    #elif defined(STM32H7) || defined(STM32L4) || defined(STM32WB)
+    #elif defined(STM32H5) || defined(STM32H7) || defined(STM32L4) || defined(STM32WB)
 
     uint32_t cfgr_clr = ADC_CFGR_CONT | ADC_CFGR_EXTEN | ADC_CFGR_RES;
     #if defined(STM32H7)
@@ -244,7 +316,7 @@ STATIC int adc_get_bits(ADC_TypeDef *adc) {
     uint32_t res = (adc->CFGR1 & ADC_CFGR1_RES) >> ADC_CFGR1_RES_Pos;
     #elif defined(STM32F4) || defined(STM32F7) || defined(STM32L1)
     uint32_t res = (adc->CR1 & ADC_CR1_RES) >> ADC_CR1_RES_Pos;
-    #elif defined(STM32G4) || defined(STM32H7) || defined(STM32L4) || defined(STM32WB)
+    #elif defined(STM32G4) || defined(STM32H5) || defined(STM32H7) || defined(STM32L4) || defined(STM32WB)
     uint32_t res = (adc->CFGR & ADC_CFGR_RES) >> ADC_CFGR_RES_Pos;
     #endif
     return adc_cr_to_bits_table[res];
@@ -311,8 +383,8 @@ STATIC void adc_config_channel(ADC_TypeDef *adc, uint32_t channel, uint32_t samp
     }
     *smpr = (*smpr & ~(7 << (channel * 3))) | sample_time << (channel * 3); // select sample time
 
-    #elif defined(STM32H7) || defined(STM32L4) || defined(STM32WB)
-    #if defined(STM32H7A3xx) || defined(STM32H7A3xxQ) || defined(STM32H7B3xx) || defined(STM32H7B3xxQ)
+    #elif defined(STM32G4) || defined(STM32H5) || defined(STM32H7) || defined(STM32L4) || defined(STM32WB)
+    #if defined(STM32G4) || defined(STM32H5) || defined(STM32H7A3xx) || defined(STM32H7A3xxQ) || defined(STM32H7B3xx) || defined(STM32H7B3xxQ)
     ADC_Common_TypeDef *adc_common = ADC12_COMMON;
     #elif defined(STM32H7)
     #if defined(ADC_VER_V5_V90)
@@ -330,12 +402,32 @@ STATIC void adc_config_channel(ADC_TypeDef *adc, uint32_t channel, uint32_t samp
     #endif
     if (channel == ADC_CHANNEL_VREFINT) {
         adc_common->CCR |= ADC_CCR_VREFEN;
+    #if defined(STM32G4)
+    } else if (channel == ADC_CHANNEL_TEMPSENSOR_ADC1) {
+        adc_common->CCR |= ADC_CCR_VSENSESEL;
+    #else
     } else if (channel == ADC_CHANNEL_TEMPSENSOR) {
         adc_common->CCR |= ADC_CCR_TSEN;
+    #endif
         adc_stabilisation_delay_us(ADC_TEMPSENSOR_DELAY_US);
     } else if (channel == ADC_CHANNEL_VBAT) {
+        #if defined(STM32G4)
+        adc_common->CCR |= ADC_CCR_VBATSEL;
+        #else
         adc_common->CCR |= ADC_CCR_VBATEN;
+        #endif
+    #if defined(STM32H5)
+    } else if (channel == ADC_CHANNEL_VDDCORE) {
+        adc->OR |= ADC_OR_OP0; // Enable Vddcore channel on ADC2
+    #endif
     }
+    #if defined(STM32G4) || defined(STM32H5)
+    // G4 and H5 use encoded literals for internal channels -> extract ADC channel for following code
+    if (__LL_ADC_IS_CHANNEL_INTERNAL(channel)) {
+        channel = __LL_ADC_CHANNEL_TO_DECIMAL_NB(channel);
+    }
+    adc->DIFSEL &= ~(1 << channel); // Set channel to Single-ended.
+    #endif
     adc->SQR1 = (channel & 0x1f) << ADC_SQR1_SQ1_Pos | (1 - 1) << ADC_SQR1_L_Pos;
     __IO uint32_t *smpr;
     if (channel <= 9) {
@@ -350,21 +442,33 @@ STATIC void adc_config_channel(ADC_TypeDef *adc, uint32_t channel, uint32_t samp
 }
 
 STATIC uint32_t adc_read_channel(ADC_TypeDef *adc) {
-    #if ADC_V2
-    adc->CR |= ADC_CR_ADSTART;
-    #else
-    adc->CR2 |= ADC_CR2_SWSTART;
+    uint32_t value;
+    #if defined(STM32G4)
+    // For STM32G4 there is errata 2.7.7, "Wrong ADC result if conversion done late after
+    // calibration or previous conversion".  According to the errata, this can be avoided
+    // by performing two consecutive ADC conversions and keeping the second result.
+    for (uint8_t i = 0; i < 2; i++)
     #endif
-    adc_wait_eoc(adc, ADC_EOC_TIMEOUT_MS);
-    uint32_t value = adc->DR;
+    {
+        #if ADC_V2
+        adc->CR |= ADC_CR_ADSTART;
+        #else
+        adc->CR2 |= ADC_CR2_SWSTART;
+        #endif
+        adc_wait_eoc(adc, ADC_EOC_TIMEOUT_MS);
+        value = adc->DR;
+    }
     return value;
 }
 
 uint32_t adc_config_and_read_u16(ADC_TypeDef *adc, uint32_t channel, uint32_t sample_time) {
-    if (channel == ADC_CHANNEL_VREF) {
+    if (channel == MACHINE_ADC_CH_VREF) {
         return 0xffff;
     }
 
+    // Map internal channel_id to STM32 ADC driver value/literal.
+    channel = adc_ll_channel(channel);
+
     // Select, configure and read the channel.
     adc_config_channel(adc, channel, sample_time);
     uint32_t raw = adc_read_channel(adc);
@@ -374,12 +478,10 @@ uint32_t adc_config_and_read_u16(ADC_TypeDef *adc, uint32_t channel, uint32_t sa
 
     // Scale raw reading to 16 bit value using a Taylor expansion (for bits <= 16).
     uint32_t bits = adc_get_bits(adc);
-    #if defined(STM32H7)
     if (bits < 8) {
         // For 6 and 7 bits
         return raw << (16 - bits) | raw << (16 - 2 * bits) | raw >> (3 * bits - 16);
     }
-    #endif
     return raw << (16 - bits) | raw >> (2 * bits - 16);
 }
 
@@ -393,7 +495,7 @@ const mp_obj_type_t machine_adc_type;
 typedef struct _machine_adc_obj_t {
     mp_obj_base_t base;
     ADC_TypeDef *adc;
-    uint32_t channel;
+    uint32_t channel; // one of machine_adc_internal_ch_t
     uint32_t sample_time;
 } machine_adc_obj_t;
 
@@ -424,20 +526,25 @@ STATIC mp_obj_t machine_adc_make_new(const mp_obj_type_t *type, size_t n_args, s
     uint32_t sample_time = ADC_SAMPLETIME_DEFAULT;
     ADC_TypeDef *adc;
     if (mp_obj_is_int(source)) {
+        channel = mp_obj_get_int(source);
         #if defined(STM32WL)
         adc = ADC;
+        #elif defined(STM32H5)
+        // on STM32H5 vbat and vddcore channels are on ADC2
+        if (channel == MACHINE_ADC_INT_CH_VBAT || channel == MACHINE_ADC_INT_CH_VDDCORE) {
+            adc = ADC2;
+        } else {
+            adc = ADC1;
+        }
         #else
         adc = ADC1;
         #endif
-        channel = mp_obj_get_int(source);
-        if (channel == ADC_CHANNEL_VREFINT
-            #if defined(STM32G4)
-            || channel == ADC_CHANNEL_TEMPSENSOR_ADC1
-            #else
-            || channel == ADC_CHANNEL_TEMPSENSOR
-            #endif
+        if (channel == MACHINE_ADC_INT_CH_VREFINT || channel == MACHINE_ADC_INT_CH_TEMPSENSOR
             #if defined(ADC_CHANNEL_VBAT)
-            || channel == ADC_CHANNEL_VBAT
+            || channel == MACHINE_ADC_INT_CH_VBAT
+            #endif
+            #if defined(ADC_CHANNEL_VDDCORE)
+            || channel == MACHINE_ADC_INT_CH_VDDCORE
             #endif
             ) {
             sample_time = ADC_SAMPLETIME_DEFAULT_INT;
@@ -488,15 +595,14 @@ MP_DEFINE_CONST_FUN_OBJ_1(machine_adc_read_u16_obj, machine_adc_read_u16);
 STATIC const mp_rom_map_elem_t machine_adc_locals_dict_table[] = {
     { MP_ROM_QSTR(MP_QSTR_read_u16), MP_ROM_PTR(&machine_adc_read_u16_obj) },
 
-    { MP_ROM_QSTR(MP_QSTR_VREF), MP_ROM_INT(ADC_CHANNEL_VREF) },
-    { MP_ROM_QSTR(MP_QSTR_CORE_VREF), MP_ROM_INT(ADC_CHANNEL_VREFINT) },
-    #if defined(STM32G4)
-    { MP_ROM_QSTR(MP_QSTR_CORE_TEMP), MP_ROM_INT(ADC_CHANNEL_TEMPSENSOR_ADC1) },
-    #else
-    { MP_ROM_QSTR(MP_QSTR_CORE_TEMP), MP_ROM_INT(ADC_CHANNEL_TEMPSENSOR) },
-    #endif
+    { MP_ROM_QSTR(MP_QSTR_VREF), MP_ROM_INT(MACHINE_ADC_CH_VREF) },
+    { MP_ROM_QSTR(MP_QSTR_CORE_VREF), MP_ROM_INT(MACHINE_ADC_INT_CH_VREFINT) },
+    { MP_ROM_QSTR(MP_QSTR_CORE_TEMP), MP_ROM_INT(MACHINE_ADC_INT_CH_TEMPSENSOR) },
     #if defined(ADC_CHANNEL_VBAT)
-    { MP_ROM_QSTR(MP_QSTR_CORE_VBAT), MP_ROM_INT(ADC_CHANNEL_VBAT) },
+    { MP_ROM_QSTR(MP_QSTR_CORE_VBAT), MP_ROM_INT(MACHINE_ADC_INT_CH_VBAT) },
+    #endif
+    #if defined(ADC_CHANNEL_VDDCORE)
+    { MP_ROM_QSTR(MP_QSTR_CORE_VDD), MP_ROM_INT(MACHINE_ADC_INT_CH_VDDCORE) },
     #endif
 };
 STATIC MP_DEFINE_CONST_DICT(machine_adc_locals_dict, machine_adc_locals_dict_table);
