@@ -71,6 +71,10 @@ class Lexer:
         ("typedef struct", re.compile(r"typedef struct$")),
         ("{", re.compile(r"{$")),
         ("}", re.compile(r"}$")),
+        (
+            "} _t",
+            re.compile(r"} *([A-Za-z0-9_]+)_t;$"),
+        ),
         (
             "} TypeDef",
             re.compile(r"} *(?P<id>[A-Z][A-Za-z0-9_]*)_(?P<global>([A-Za-z0-9_]+)?)TypeDef;$"),
@@ -157,7 +161,7 @@ def parse_file(filename):
                     for i in range(int(d["array"])):
                         regs.append((reg + str(i), offset + i * bits // 8, bits, comment))
                 m = lexer.next_match()
-            if m[0] == "}":
+            if m[0] in ("}", "} _t"):
                 pass
             elif m[0] == "} TypeDef":
                 d = m[1].groupdict()
@@ -265,7 +269,6 @@ def main():
         reg_defs["GPIO"].append(["BSRRL", 0x18, 16, "legacy register"])
         reg_defs["GPIO"].append(["BSRRH", 0x1A, 16, "legacy register"])
 
-    modules = []
     needed_qstrs = set()
     needed_mpzs = set()
 
@@ -316,10 +319,10 @@ def main():
     print("")
 
     with open(args.qstr_filename, "wt") as qstr_file:
-        print("#if MICROPY_PY_STM", file=qstr_file)
+        print("#if MICROPY_PY_STM_CONST", file=qstr_file)
         for qstr in sorted(needed_qstrs):
             print("Q({})".format(qstr), file=qstr_file)
-        print("#endif // MICROPY_PY_STM", file=qstr_file)
+        print("#endif // MICROPY_PY_STM_CONST", file=qstr_file)
 
     with open(args.mpz_filename, "wt") as mpz_file:
         for mpz in sorted(needed_mpzs):
